@@ -63,6 +63,7 @@ ImportCertificateDialog::ImportCertificateDialog( QWidget *parent, const char* c
 	pathToOpenSSL = GlobalConfig->pathToOpenssl;
 	main->RacoonCertificatePathUrlrequester->setMode( 2 ); // directory
 	main->FilenameUrlrequester->setFilter( "*.p12" );
+	askpasscount=0;
 
 	connect (main->ImporttypeComboBox, SIGNAL(activated(int)), this, SLOT(typeToggeled(int)));
 	typeToggeled( main->ImporttypeComboBox->currentItem() );
@@ -101,6 +102,7 @@ void ImportCertificateDialog::accept()
 		KMessageBox::sorry( 0, i18n( "File name cannot be empty!" ), i18n( "Empty File Name" ) );
 		importOk = false;
 		importSuccess=false;
+		GlobalConfig->appPointer->restoreOverrideCursor();
 		return;
 	}
 
@@ -229,12 +231,13 @@ void ImportCertificateDialog::accept()
 // 		return;
 // 	}
 
-	if ( main->ImporttypeComboBox->currentItem() == pkcs12_freeswan )
+	if ( main->ImporttypeComboBox->currentItem() == pkcs12_freeswan || main->ImporttypeComboBox->currentItem() == pkcs12_openvpn )
 	{
 		if ( main->PrivateKeyPasswordEdit->text().isEmpty() || main->PrivateKeyPasswordEdit->text().length() < 4  )
 		{
 			importOk = false;
 			KMessageBox::sorry( 0, i18n( "Private key password field can not be empty or less than 4 characters!" ), i18n( "Password Empty or Too Short" ) );
+			GlobalConfig->appPointer->restoreOverrideCursor();
 			return;
 		}
 
@@ -247,7 +250,7 @@ void ImportCertificateDialog::accept()
 		}
 		if ( importOk )
 		{
-			if ( main->PrivateKeyPasswordEdit->text() != main->PrivateKeyAgainPasswordEdit->text() )
+			if ( QString(main->PrivateKeyPasswordEdit->password()) != QString(main->PrivateKeyAgainPasswordEdit->password()) )
 			{
 				importOk = false;
 				KMessageBox::sorry( 0, i18n( "Private key passwords does not match!" ), i18n( "Passwords Do Not Match" ) );
@@ -256,7 +259,7 @@ void ImportCertificateDialog::accept()
 			}
 			else
 			{
-				privatekeypassword= main->PrivateKeyAgainPasswordEdit->text();
+				privatekeypassword= QString(main->PrivateKeyAgainPasswordEdit->password());
 			}
 		}
 	}
@@ -562,7 +565,8 @@ void ImportCertificateDialog::doPrivateKey()
 {
 	if ( ImportProcess->normalExit () )
 	{
-		//GlobalConfig->appendLogEntry("[dbg]: doPrivateKey()",GlobalConfig->debug);
+		if (GlobalConfig->KvpncDebugLevel > 1)
+			GlobalConfig->appendLogEntry("[dbg]: doPrivateKey()",GlobalConfig->debug);
 		disconnect( ImportProcess, SIGNAL( processExited () ), this, SLOT( doPrivateKey() ) );
 		/*
 		openssl pkcs12 -in cert.p12 -nocerts -out /etc/racoon/certs/ca_company.pem
@@ -576,9 +580,16 @@ void ImportCertificateDialog::doPrivateKey()
 		ImportProcess->addArgument( "-out" );
 
 
-		ImportProcess->addArgument( certpath+"/private/" + certName + ".pem" ); // warning: static
+		if ( main->ImporttypeComboBox->currentItem() == pkcs12_freeswan)
+			ImportProcess->addArgument( certpath+"/private/" + certName + ".pem" ); // warning: static
+		else if ( main->ImporttypeComboBox->currentItem() == pkcs12_openvpn)
+			ImportProcess->addArgument( certpath+"/" + certName + ".pem" ); // warning: static
+
 		if (!certpath.isEmpty())
-			CertFullPath=certpath+"/private/" + certName + ".pem";
+			if ( main->ImporttypeComboBox->currentItem() == pkcs12_freeswan)
+				CertFullPath=certpath+"/private/" + certName + ".pem";
+			if ( main->ImporttypeComboBox->currentItem() == pkcs12_openvpn)
+				CertFullPath=certpath+"/" + certName + ".pem";
 
 		connect( ImportProcess, SIGNAL( processExited () ), this, SLOT( exit() ) );
 
@@ -595,7 +606,7 @@ void ImportCertificateDialog::doPrivateKey()
 		else
 		{
 			doExit=true;
-			privatekeypassword = main->PrivateKeyPasswordEdit->text();
+			privatekeypassword = main->PrivateKeyPasswordEdit->password();
 			if (GlobalConfig->KvpncDebugLevel > 0 )
 				GlobalConfig->appendLogEntry ( i18n( "Certificate import: private key successful extracted." ),GlobalConfig->debug );
 		}
@@ -671,10 +682,11 @@ void ImportCertificateDialog::doLink()
 		else
 		{
 			ImportProcess->addArgument( certpath + "/ca_" + certName + ".pem" );
-			ImportProcess->addArgument( certpath + certHash + ".0" );
+			ImportProcess->addArgument( certpath +"/"+ certHash + ".0" );
 		}
 
-		if ( main->ImporttypeComboBox->currentItem() == pkcs12_freeswan )
+		if ( main->ImporttypeComboBox->currentItem() == pkcs12_freeswan ||
+				main->ImporttypeComboBox->currentItem() == pkcs12_openvpn )
 			connect( ImportProcess, SIGNAL( processExited () ), this, SLOT( doPrivateKey() ) );
 		else
 			connect( ImportProcess, SIGNAL( processExited () ), this, SLOT( exit() ) );
@@ -703,39 +715,58 @@ void ImportCertificateDialog::exit()
 	// 		importOk=true;
 	if ( main->ImporttypeComboBox->currentItem() == pkcs12_freeswan )
 		Utils(GlobalConfig,this).doChmod(certpath+"/private/" + certName + ".pem","go-rwx");
+
+	if ( main->ImporttypeComboBox->currentItem() == pkcs12_openvpn )
+		Utils(GlobalConfig,this).doChmod(certpath+"/" + certName + ".pem","go-rwx");
 	done=true;
 	canAccept();
 }
 
 void ImportCertificateDialog::readFromStdout_import()
 {
-	QString line = QString( ImportProcess->readStdout() );
-
-	if (GlobalConfig->KvpncDebugLevel > 0 )
-		GlobalConfig->appendLogEntry ( i18n( "Certificate import stdout: %1" ).arg( line ),GlobalConfig->debug );
-	if ( doHash )
-	{
-		certHash = line.left( line.length() - 1 );
-		doHash = false;
-		//		LogOutput->append( "<font color=\"red\">[std]: hash" + line + "</font>" );
-	}
-
-	if ( line.find( "Enter Import Pass" , 0, FALSE ) > -1 )
-	{
-		if (GlobalConfig->KvpncDebugLevel > 0 )
-			GlobalConfig->appendLogEntry ( i18n( "Certificate import: %1 was requested, send it..." ).arg(i18n(" import password")),GlobalConfig->debug );
-
-		ImportProcess->writeToStdin( importpassword + "\n" );
-	}
-
-	if ( line.find( "Enter PEM pass phrase" , 0, FALSE ) > -1 )
+	
+// 	QString line = QString( ImportProcess->readStdout() );
+	while (ImportProcess->canReadLineStdout() )
 	{
+		QString line = QString( ImportProcess->readLineStdout() );
 
 		if (GlobalConfig->KvpncDebugLevel > 0 )
-			GlobalConfig->appendLogEntry ( i18n( "Certificate import: %1 was requested, send it..." ).arg(i18n("private key password")),GlobalConfig->debug );
-		ImportProcess->writeToStdin( main->PrivateKeyPasswordEdit->text() + "\n" );
+			GlobalConfig->appendLogEntry ( i18n( "Certificate import stdout: %1" ).arg( line ),GlobalConfig->debug );
+		if ( doHash )
+		{
+			certHash = line.left( line.length() - 1 );
+			doHash = false;
+			//		LogOutput->append( "<font color=\"red\">[std]: hash" + line + "</font>" );
+		}
+	
+		if ( line.find( "Enter Import Password" , 0, FALSE ) > -1 )
+		{
+			if (GlobalConfig->KvpncDebugLevel > 0 )
+				GlobalConfig->appendLogEntry ( i18n( "Certificate import: %1 was requested, send it..." ).arg(i18n(" import password")),GlobalConfig->debug );
+	
+			if (main->ImportPasswordEdit->text().isEmpty())
+				ImportProcess->writeToStdin( "\n" );
+			else
+				ImportProcess->writeToStdin( main->ImportPasswordEdit->text() + "\n" );
+		}
+	
+		if ( line.find( "Enter PEM pass phrase" , 0, FALSE ) > -1 )
+		{
+	
+			if (askpasscount < 10)
+			{
+				if (GlobalConfig->KvpncDebugLevel > 0 )
+					GlobalConfig->appendLogEntry ( i18n( "Certificate import: %1 was requested, send it..." ).arg(i18n("private key password")),GlobalConfig->debug );
+				ImportProcess->writeToStdin( QString(main->PrivateKeyPasswordEdit->password()) + "\n" );
+				askpasscount+=1;
+			}
+			else
+			{
+				// whats happen???
+	// 			doExit=true;
+			}
+		}
 	}
-
 }
 
 void ImportCertificateDialog::readFromStderr_import()
@@ -771,7 +802,7 @@ void ImportCertificateDialog::readFromStderr_import()
 
 		if (GlobalConfig->KvpncDebugLevel > 0 )
 			GlobalConfig->appendLogEntry ( i18n( "Certificate import: password was requested, send it..." ).arg(i18n("private key password")),GlobalConfig->debug );
-		ImportProcess->writeToStdin( main->PrivateKeyPasswordEdit->text() + "\n" );
+		ImportProcess->writeToStdin( QString(main->PrivateKeyPasswordEdit->password()) + "\n" );
 	}
 
 	// 	if ( line.contains( "Verifying - Enter PEM pass phrase" ) > 0 ) {
@@ -799,7 +830,8 @@ void ImportCertificateDialog::readFromStderr_import()
 
 void ImportCertificateDialog::typeToggeled( int type )
 {
-	//GlobalConfig->appendLogEntry( QString("type: "+QString().setNum(type)),GlobalConfig->info);
+	if (GlobalConfig->KvpncDebugLevel > 0 )
+		GlobalConfig->appendLogEntry( QString("type: "+QString().setNum(type)),GlobalConfig->debug);
 	if ( type == pkcs12_freeswan )
 	{
 		certpath = "/etc/ipsec.d/certs";
@@ -841,7 +873,7 @@ void ImportCertificateDialog::typeToggeled( int type )
 		main->RacoonCertificatePathUrlrequester->setEnabled( false );
 		main->ImportPasswordEdit->setEnabled( true );
 		main->P12GroupBox->setEnabled( true );
-		main->FreeswanGroupBox->setEnabled( false );
+		main->FreeswanGroupBox->setEnabled( true );
 		main->FilenameUrlrequester->setFilter( "*.p12" );
 	}
 	main->RacoonCertificatePathUrlrequester->setURL(certpath);
