@@ -39,6 +39,7 @@
 #include <qtextstream.h>
 #include <qcheckbox.h>
 #include <string>
+#include <qprocess.h>
 
 #include <iostream>
 
@@ -215,14 +216,23 @@ void ImportOpenvpnProfileDialog::canAccept() {
 	QFile OpenvpnConfigFile( filename );
 	QTextStream stream ( &OpenvpnConfigFile );
 
+	QString importprefix = QFileInfo ( filename ).dirPath();
+	
+	if ( config->KvpncDebugLevel > 2 )
+		config->appendLogEntry( i18n( "OpenVPN import: import prefix: %1" ).arg( importprefix ), config->debug );
+
+	QString certprefix = locateLocal ( "data", "kvpnc" );
+
 	if ( OpenvpnConfigFile.open( IO_ReadOnly ) ) {
 		VpnAccountData::ConnectionType ConnType = VpnAccountData::openvpn;
 		acc = new VpnAccountData ( ConnType, "" );
-		acc->setName( filename.section( '/', -1, -1 ).section( '.', -2, -2 ) ); // /home/user/openvpn/CompanyABC.conf -> CompanyABC
+		QString profilename = filename.section( '/', -1, -1 ).section( '.', -2, -2 );
+		acc->setName( profilename ); // /home/user/openvpn/CompanyABC.conf -> CompanyABC
 		acc->setDescription( i18n( "Imported from %1" ).arg( filename.section( '/', -1, -1 ) ) );
 		acc->setLocalPort( 1194 );
 		acc->setAuthWithUsernameAndPassword( false );
 		acc->setRemoteNetMask( "24" );
+		acc->setAuthenticationAlgorithm("MD5");
 
 		acc->setUseTlsAuth( false );
 		// 		acc->setUseConnectionStatusCheck( false );
@@ -302,28 +312,101 @@ void ImportOpenvpnProfileDialog::canAccept() {
 				}
 				else if ( line.startsWith( "cert " ) ) {
 					acc->setAuthType( VpnAccountData::cert );
-					// 					if (CertPath.isEmpty())
-					acc->setX509Certificate( line.section( ' ', 1 ) );
-					// 					else
-					// 						acc->setX509Certificate( CertPath+"/"+line.section(' ',1,1));
+					QString cert = line.section( ' ', 1 ).stripWhiteSpace();
+
+					if ( config->KvpncDebugLevel > 2 )
+						config->appendLogEntry( i18n( "OpenVPN import: cert file: %1" ).arg( importprefix+"/"+cert), config->debug );
+
+					if ( QFile( QString( importprefix+"/"+cert )).exists())
+					{
+						// copy to ~/.kde/share/apps/kvpnc/ and rename it
+					if ( config->KvpncDebugLevel > 0 )
+						config->appendLogEntry( i18n( "OpenVPN import: copy %1 to %2" ).arg(QString( importprefix+"/"+cert )).arg(QString(certprefix+"/"+"openvpn_cert_"+profilename+"_"+cert)), config->debug );
+						QProcess CopyProcess( this );
+						CopyProcess.addArgument("cp");
+						CopyProcess.addArgument(QString( importprefix+"/"+cert ));
+						CopyProcess.addArgument(QString(certprefix+"/"+"openvpn_cert_"+profilename+"_"+cert));
+						if (CopyProcess.start())
+							acc->setX509Certificate( QString(certprefix+"/"+"openvpn_cert_"+profilename+"_"+cert) );
+						else
+						{
+							config->appendLogEntry( i18n( "Could not start %1!" ).arg( "cp" ), config->error );
+							acc->setX509Certificate( cert );
+						}
+					}
+					else
+					{
+						// we use the given path
+						acc->setX509Certificate( cert );
+					}
+
 					if ( config->KvpncDebugLevel > 0 )
 						config->appendLogEntry( i18n( "OpenVPN import: certificate: %1" ).arg( acc->getX509Certificate() ), config->debug );
 				}
 				else if ( line.startsWith( "key " ) ) {
-					acc->setAuthType( VpnAccountData::cert );
-					// 					if (CertPath.isEmpty())
-					acc->setPrivateKey( line.section( ' ', 1 ) );
-					// 					else
-					// 						acc->setPrivateKey(CertPath+"/"+line.section(' ',1,1));
+
+					QString privatekey = line.section( ' ', 1 ).stripWhiteSpace();
+
+					if ( config->KvpncDebugLevel > 2 )
+						config->appendLogEntry( i18n( "OpenVPN import: private key file: %1" ).arg( importprefix+"/"+privatekey ), config->debug );
+
+					if ( QFile( QString( importprefix+"/"+privatekey )).exists())
+					{
+						// copy to ~/.kde/share/apps/kvpnc/ and rename it
+					if ( config->KvpncDebugLevel > 0 )
+						config->appendLogEntry( i18n( "OpenVPN import: copy %1 to %2" ).arg(QString( importprefix+"/"+privatekey )).arg(QString(certprefix+"/"+"openvpn_privatekey_"+profilename+"_"+privatekey)), config->debug );
+						QProcess CopyProcess( this );
+						CopyProcess.addArgument("cp");
+						CopyProcess.addArgument(QString( importprefix+"/"+privatekey ));
+						CopyProcess.addArgument(QString(certprefix+"/"+"openvpn_privatekey_"+profilename+"_"+privatekey));
+						if (CopyProcess.start())
+							acc->setPrivateKey( QString(certprefix+"/"+"openvpn_privatekey_"+profilename+"_"+privatekey) );
+						else
+						{
+							config->appendLogEntry( i18n( "Could not start %1!" ).arg( "cp" ), config->error );
+							acc->setPrivateKey( privatekey );
+						}
+					}
+					else
+					{
+						// we use the given path
+						acc->setPrivateKey( privatekey );
+					}
+
 					if ( config->KvpncDebugLevel > 0 )
 						config->appendLogEntry( i18n( "OpenVPN import: PSK in file: %1" ).arg( acc->getPrivateKey() ), config->debug );
 				}
 				else if ( line.startsWith( "ca " ) ) {
 					acc->setAuthType( VpnAccountData::cert );
-					// 					if (CertPath.isEmpty())
-					acc->setCaCertificate( line.section( ' ', 1 ) );
-					// 					else
-					// 						acc->setCaCertificate(CertPath+"/"+line.section(' ',1,1));
+
+					QString ca = line.section( ' ', 1 );
+
+					if ( config->KvpncDebugLevel > 2 )
+						config->appendLogEntry( i18n( "OpenVPN import: ca file: %1" ).arg( importprefix+"/"+ca ), config->debug );
+
+					if ( QFile( QString( importprefix+"/"+ca )).exists())
+					{
+						// copy to ~/.kde/share/apps/kvpnc/ and rename it
+					if ( config->KvpncDebugLevel > 0 )
+						config->appendLogEntry( i18n( "OpenVPN import: copy %1 to %2" ).arg(QString( importprefix+"/"+ca )).arg(QString(certprefix+"/"+"openvpn_privatekey_"+profilename+"_"+ca)), config->debug );
+						QProcess CopyProcess( this );
+						CopyProcess.addArgument("cp");
+						CopyProcess.addArgument(QString( importprefix+"/"+ca ));
+						CopyProcess.addArgument(QString(certprefix+"/"+"openvpn_privatekey_"+profilename+"_"+ca));
+						if (CopyProcess.start())
+							acc->setCaCertificate( QString(certprefix+"/"+"openvpn_privatekey_"+profilename+"_"+ca) );
+						else
+						{
+							config->appendLogEntry( i18n( "Could not start %1!" ).arg( "cp" ), config->error );
+							acc->setCaCertificate( ca );
+						}
+					}
+					else
+					{
+						// we use the given path
+						acc->setCaCertificate( ca );
+					}
+
 					if ( config->KvpncDebugLevel > 0 )
 						config->appendLogEntry( i18n( "OpenVPN import: CA certificate: %1" ).arg( acc->getCaCertificate() ), config->debug );
 				}
@@ -347,7 +430,35 @@ void ImportOpenvpnProfileDialog::canAccept() {
 				}
 				else if ( line.startsWith( "tls-auth " ) ) {
 					acc->setUseTlsAuth( true );
-					acc->setTlsAuthFile( line.section( ' ', 1, 1 ) );
+
+				QString tlsauth = line.section( ' ', 1,1 );
+
+					if ( config->KvpncDebugLevel > 2 )
+						config->appendLogEntry( i18n( "OpenVPN import: TLS auth file: %1" ).arg( importprefix+"/"+tlsauth ), config->debug );
+
+					if ( QFile( QString( importprefix+"/"+tlsauth )).exists())
+					{
+						// copy to ~/.kde/share/apps/kvpnc/ and rename it
+					if ( config->KvpncDebugLevel > 0 )
+						config->appendLogEntry( i18n( "OpenVPN import: copy %1 to %2" ).arg(QString( importprefix+"/"+tlsauth )).arg(QString(certprefix+"/"+"openvpn_privatekey_"+profilename+"_"+tlsauth)), config->debug );
+						QProcess CopyProcess( this );
+						CopyProcess.addArgument("cp");
+						CopyProcess.addArgument(QString( importprefix+"/"+tlsauth ));
+						CopyProcess.addArgument(QString(certprefix+"/"+"openvpn_tlsauth_"+profilename+"_"+tlsauth));
+						if (CopyProcess.start())
+							acc->setTlsAuthFile( QString(certprefix+"/"+"openvpn_tlsauth_"+profilename+"_"+tlsauth) );
+						else
+						{
+							config->appendLogEntry( i18n( "Could not start %1!" ).arg( "cp" ), config->error );
+							acc->setTlsAuthFile( tlsauth );
+						}
+					}
+					else
+					{
+						// we use the given path
+						acc->setTlsAuthFile( tlsauth );
+					}
+
 					if ( config->KvpncDebugLevel > 0 )
 						config->appendLogEntry( i18n( "OpenVPN import: use TLS auth" ), config->debug );
 				}
@@ -408,128 +519,141 @@ void ImportOpenvpnProfileDialog::canAccept() {
 						config->appendLogEntry( i18n( "OpenVPN import: certificate prefix found: %1" ).arg( CertPath ), config->debug );
 				}
 				else if ( line.startsWith( "tls-remote" ) ) {
-					acc->setUseTlsRemoteHost( true );
-					acc->setTlsRemoteHost( line.section( ' ', 1, 1 ) );
+					if (!line.section( ' ', 1, 1 ).isEmpty())
+					{
+						acc->setUseTlsRemoteHost( true );
+						acc->setTlsRemoteHost( line.section( ' ', 1, 1 ) );
+						if ( config->KvpncDebugLevel > 0 )
+							config->appendLogEntry( i18n( "OpenVPN import: use tls remote host: %1" ).arg( acc->getTlsRemoteHost() ), config->debug );
+					}
+					else
+					{
+						if ( config->KvpncDebugLevel > 0 )
+							config->appendLogEntry( i18n( "OpenVPN import: tls remote host option was specified but server is empty, disabling TLS remote host." ), config->debug );
+					}
+				}
+				else if ( line.startsWith( "float" ) ) {
+					acc->setAllowIpAddressChangeOfPeer( true );
 					if ( config->KvpncDebugLevel > 0 )
-						config->appendLogEntry( i18n( "OpenVPN import: use tls remote host: %1" ).arg( acc->getTlsRemoteHost() ), config->debug );
+						config->appendLogEntry( i18n( "OpenVPN import: allow IP address change of peer (for DHCP)" ), config->debug );
 				}
 
 			}
 		}
 
-		if ( acc->getCertPath().isEmpty() ) {
-			// 						acc->setCertPath("/etc/openvpn/");
-			acc->setCertPath( locateLocal ( "data", "kvpnc" ) );
-			CertPath = acc->getCertPath();
-		}
-		// 					/* we need to put the prefix to all cert paths */
-		if ( !acc->getCaCertificate().isEmpty() && !acc->getCaCertificate().startsWith( CertPath ) ) {
-			QString prefix = "./";
-			if ( !filename.find( '/' ) > -1 )
-				prefix = QString( filename.section( '/', 0, -2 ) + "/" );
-
-			QFile CaCertFileSrc ( prefix + acc->getCaCertificate() );
-			QFile CaCertFileDst ( QString( CertPath + "/openvpnimport_" + acc->getCaCertificate() ) );
-			acc->setCaCertificate( QString( CertPath + "/openvpnimport_" + acc->getCaCertificate() ) );
-
-			QTextStream CaCertDstStream( &CaCertFileDst );
-
-			if ( config->KvpncDebugLevel > 1 )
-				config->appendLogEntry( i18n( "OpenVPN import: try to copy ca cert file %1 to %2." ).arg( CaCertFileSrc.name() ).arg( CaCertFileDst.name() ), config->debug );
-
-
-			if ( CaCertFileSrc.open( IO_ReadOnly ) ) {
-				if ( CaCertFileDst.open( IO_WriteOnly ) ) {
-					QString CaCertFileString = QString( CaCertFileSrc.readAll() );
-					CaCertDstStream << CaCertFileString;
-					CaCertFileDst.close();
-					if ( config->KvpncDebugLevel > 1 )
-						config->appendLogEntry( i18n( "OpenVPN import: ca cert file %1 copied to %2." ).arg( CaCertFileSrc.name() ).arg( CaCertFileDst.name() ), config->debug );
-				} else {
-					if ( config->KvpncDebugLevel > 2 )
-						config->appendLogEntry( i18n( "OpenVPN import: ca cert file %1 could not open for writing." ).arg( CaCertFileDst.name() ), config->debug );
-
-					//								return;
-				}
-				CaCertFileDst.close();
-			} else {
-				if ( config->KvpncDebugLevel > 2 )
-					config->appendLogEntry( i18n( "OpenVPN import: ca cert file %1 could not open for reading." ).arg( CaCertFileSrc.name() ), config->debug );
-
-				//							return;
-			}
-		}
-		if ( !acc->getX509Certificate().isEmpty() && !acc->getX509Certificate().startsWith( CertPath ) ) {
-			QString prefix = "./";
-			if ( !filename.find( '/' ) > -1 )
-				prefix = QString( filename.section( '/', 0, -2 ) + "/" );
-
-			QFile X509CertFileSrc ( prefix + acc->getX509Certificate() );
-			QFile X509CertFileDst ( QString( CertPath + "/openvpnimport_" + acc->getX509Certificate() ) );
-			acc->setX509Certificate( QString( CertPath + "/openvpnimport_" + acc->getX509Certificate() ) );
-
-			QTextStream X509CertDstStream( &X509CertFileDst );
-
-			if ( config->KvpncDebugLevel > 1 )
-				config->appendLogEntry( i18n( "OpenVPN import: try to copy pre shared key file %1 to %2." ).arg( X509CertFileSrc.name() ).arg( X509CertFileDst.name() ), config->debug );
-
-
-			if ( X509CertFileSrc.open( IO_ReadOnly ) ) {
-				if ( X509CertFileDst.open( IO_WriteOnly ) ) {
-					QString X509CertFileString = QString( X509CertFileSrc.readAll() );
-					X509CertDstStream << X509CertFileString;
-					X509CertFileDst.close();
-					if ( config->KvpncDebugLevel > 2 )
-						config->appendLogEntry( i18n( "OpenVPN import: X509 cert file %1 copied to %2." ).arg( X509CertFileSrc.name() ).arg( X509CertFileDst.name() ), config->debug );
-				} else {
-					if ( config->KvpncDebugLevel > 2 )
-						config->appendLogEntry( i18n( "OpenVPN import: X509 cert file %1 could not open for writing." ).arg( X509CertFileDst.name() ), config->debug );
-
-					//								return;
-				}
-				X509CertFileDst.close();
-			} else {
-				if ( config->KvpncDebugLevel > 2 )
-					config->appendLogEntry( i18n( "OpenVPN import: X509 cert file %1 could not open for reading." ).arg( X509CertFileSrc.name() ), config->debug );
-
-				//							return;
-			}
-		}
-		if ( !acc->getPreSharedKey().isEmpty() && !acc->getPreSharedKey().startsWith( CertPath ) ) {
-			QString prefix = "./";
-			if ( !filename.find( '/' ) > -1 )
-				prefix = QString( filename.section( '/', 0, -2 ) + "/" );
-
-			QFile PreSharedKeyFileSrc ( QString( prefix + acc->getPreSharedKey() ) );
-			QFile PreSharedKeyFileDst ( QString( CertPath + "/openvpnimport_" + acc->getPreSharedKey() ) );
-			acc->setPreSharedKey( QString( CertPath + "/openvpnimport_" + acc->getPreSharedKey() ) );
-
-			QTextStream PreSharedKeyDstStream( &PreSharedKeyFileDst );
-
-			if ( config->KvpncDebugLevel > 1 )
-				config->appendLogEntry( i18n( "OpenVPN import: try to copy pre shared key file %1 to %2." ).arg( PreSharedKeyFileSrc.name() ).arg( PreSharedKeyFileDst.name() ), config->debug );
-
-			if ( PreSharedKeyFileSrc.open( IO_ReadOnly ) ) {
-				if ( PreSharedKeyFileDst.open( IO_WriteOnly ) ) {
-					QString PreSharedKeyFileString = QString( PreSharedKeyFileSrc.readAll() );
-					PreSharedKeyDstStream << PreSharedKeyFileString;
-					PreSharedKeyFileDst.close();
-					if ( config->KvpncDebugLevel > 2 )
-						config->appendLogEntry( i18n( "OpenVPN import: pre shared key file %1 copied to %2." ).arg( PreSharedKeyFileSrc.name() ).arg( PreSharedKeyFileDst.name() ), config->debug );
-				} else {
-					if ( config->KvpncDebugLevel > 2 )
-						config->appendLogEntry( i18n( "OpenVPN import: pre shared key file %1 could not open for writing." ).arg( PreSharedKeyFileDst.name() ), config->debug );
-
-					//								return;
-				}
-				PreSharedKeyFileDst.close();
-			} else {
-				if ( config->KvpncDebugLevel > 2 )
-					config->appendLogEntry( i18n( "OpenVPN import: pre shared key file %1 could not open for reading." ).arg( PreSharedKeyFileSrc.name() ), config->debug );
-
-				//							return;
-			}
-		}
+// 		if ( acc->getCertPath().isEmpty() ) {
+// 			// 						acc->setCertPath("/etc/openvpn/");
+// 			acc->setCertPath( locateLocal ( "data", "kvpnc" ) );
+// 			CertPath = acc->getCertPath();
+// 		}
+// 		// 					/* we need to put the prefix to all cert paths */
+// 		if ( !acc->getCaCertificate().isEmpty() && !acc->getCaCertificate().startsWith( CertPath ) ) {
+// 			QString prefix = "./";
+// 			if ( !filename.find( '/' ) > -1 )
+// 				prefix = QString( filename.section( '/', 0, -2 ) + "/" );
+// 
+// 			QFile CaCertFileSrc ( prefix + acc->getCaCertificate() );
+// 			QFile CaCertFileDst ( QString( CertPath + "/openvpnimport_" + acc->getCaCertificate() ) );
+// 			acc->setCaCertificate( QString( CertPath + "/openvpnimport_" + acc->getCaCertificate() ) );
+// 
+// 			QTextStream CaCertDstStream( &CaCertFileDst );
+// 
+// 			if ( config->KvpncDebugLevel > 1 )
+// 				config->appendLogEntry( i18n( "OpenVPN import: try to copy ca cert file %1 to %2." ).arg( CaCertFileSrc.name() ).arg( CaCertFileDst.name() ), config->debug );
+// 
+// 
+// 			if ( CaCertFileSrc.open( IO_ReadOnly ) ) {
+// 				if ( CaCertFileDst.open( IO_WriteOnly ) ) {
+// 					QString CaCertFileString = QString( CaCertFileSrc.readAll() );
+// 					CaCertDstStream << CaCertFileString;
+// 					CaCertFileDst.close();
+// 					if ( config->KvpncDebugLevel > 1 )
+// 						config->appendLogEntry( i18n( "OpenVPN import: ca cert file %1 copied to %2." ).arg( CaCertFileSrc.name() ).arg( CaCertFileDst.name() ), config->debug );
+// 				} else {
+// 					if ( config->KvpncDebugLevel > 2 )
+// 						config->appendLogEntry( i18n( "OpenVPN import: ca cert file %1 could not open for writing." ).arg( CaCertFileDst.name() ), config->debug );
+// 
+// 					//								return;
+// 				}
+// 				CaCertFileDst.close();
+// 			} else {
+// 				if ( config->KvpncDebugLevel > 2 )
+// 					config->appendLogEntry( i18n( "OpenVPN import: ca cert file %1 could not open for reading." ).arg( CaCertFileSrc.name() ), config->debug );
+// 
+// 				//							return;
+// 			}
+// 		}
+// 		if ( !acc->getX509Certificate().isEmpty() && !acc->getX509Certificate().startsWith( CertPath ) ) {
+// 			QString prefix = "./";
+// 			if ( !filename.find( '/' ) > -1 )
+// 				prefix = QString( filename.section( '/', 0, -2 ) + "/" );
+// 
+// 			QFile X509CertFileSrc ( prefix + acc->getX509Certificate() );
+// 			QFile X509CertFileDst ( QString( CertPath + "/openvpnimport_" + acc->getX509Certificate() ) );
+// 			acc->setX509Certificate( QString( CertPath + "/openvpnimport_" + acc->getX509Certificate() ) );
+// 
+// 			QTextStream X509CertDstStream( &X509CertFileDst );
+// 
+// 			if ( config->KvpncDebugLevel > 1 )
+// 				config->appendLogEntry( i18n( "OpenVPN import: try to copy pre shared key file %1 to %2." ).arg( X509CertFileSrc.name() ).arg( X509CertFileDst.name() ), config->debug );
+// 
+// 
+// 			if ( X509CertFileSrc.open( IO_ReadOnly ) ) {
+// 				if ( X509CertFileDst.open( IO_WriteOnly ) ) {
+// 					QString X509CertFileString = QString( X509CertFileSrc.readAll() );
+// 					X509CertDstStream << X509CertFileString;
+// 					X509CertFileDst.close();
+// 					if ( config->KvpncDebugLevel > 2 )
+// 						config->appendLogEntry( i18n( "OpenVPN import: X509 cert file %1 copied to %2." ).arg( X509CertFileSrc.name() ).arg( X509CertFileDst.name() ), config->debug );
+// 				} else {
+// 					if ( config->KvpncDebugLevel > 2 )
+// 						config->appendLogEntry( i18n( "OpenVPN import: X509 cert file %1 could not open for writing." ).arg( X509CertFileDst.name() ), config->debug );
+// 
+// 					//								return;
+// 				}
+// 				X509CertFileDst.close();
+// 			} else {
+// 				if ( config->KvpncDebugLevel > 2 )
+// 					config->appendLogEntry( i18n( "OpenVPN import: X509 cert file %1 could not open for reading." ).arg( X509CertFileSrc.name() ), config->debug );
+// 
+// 				//							return;
+// 			}
+// 		}
+// 		if ( !acc->getPreSharedKey().isEmpty() && !acc->getPreSharedKey().startsWith( CertPath ) ) {
+// 			QString prefix = "./";
+// 			if ( !filename.find( '/' ) > -1 )
+// 				prefix = QString( filename.section( '/', 0, -2 ) + "/" );
+// 
+// 			QFile PreSharedKeyFileSrc ( QString( prefix + acc->getPreSharedKey() ) );
+// 			QFile PreSharedKeyFileDst ( QString( CertPath + "/openvpnimport_" + acc->getPreSharedKey() ) );
+// 			acc->setPreSharedKey( QString( CertPath + "/openvpnimport_" + acc->getPreSharedKey() ) );
+// 
+// 			QTextStream PreSharedKeyDstStream( &PreSharedKeyFileDst );
+// 
+// 			if ( config->KvpncDebugLevel > 1 )
+// 				config->appendLogEntry( i18n( "OpenVPN import: try to copy pre shared key file %1 to %2." ).arg( PreSharedKeyFileSrc.name() ).arg( PreSharedKeyFileDst.name() ), config->debug );
+// 
+// 			if ( PreSharedKeyFileSrc.open( IO_ReadOnly ) ) {
+// 				if ( PreSharedKeyFileDst.open( IO_WriteOnly ) ) {
+// 					QString PreSharedKeyFileString = QString( PreSharedKeyFileSrc.readAll() );
+// 					PreSharedKeyDstStream << PreSharedKeyFileString;
+// 					PreSharedKeyFileDst.close();
+// 					if ( config->KvpncDebugLevel > 2 )
+// 						config->appendLogEntry( i18n( "OpenVPN import: pre shared key file %1 copied to %2." ).arg( PreSharedKeyFileSrc.name() ).arg( PreSharedKeyFileDst.name() ), config->debug );
+// 				} else {
+// 					if ( config->KvpncDebugLevel > 2 )
+// 						config->appendLogEntry( i18n( "OpenVPN import: pre shared key file %1 could not open for writing." ).arg( PreSharedKeyFileDst.name() ), config->debug );
+// 
+// 					//								return;
+// 				}
+// 				PreSharedKeyFileDst.close();
+// 			} else {
+// 				if ( config->KvpncDebugLevel > 2 )
+// 					config->appendLogEntry( i18n( "OpenVPN import: pre shared key file %1 could not open for reading." ).arg( PreSharedKeyFileSrc.name() ), config->debug );
+// 
+// 				//							return;
+// 			}
+// 		}
 
 		OpenvpnConfigFile.close();
 		importOk = true;
@@ -550,17 +674,19 @@ void ImportOpenvpnProfileDialog::canAccept() {
 			p12file = Pkcs12CertFile;
 
 		dlg.main->ImporttypeComboBox->setCurrentItem( ImportCertificateDialog::pkcs12_openvpn );
+		dlg.typeToggeled( ImportCertificateDialog::pkcs12_openvpn );
 		dlg.main->ImporttypeComboBox->setEnabled( false );
 		// 		dlg.main->RacoonCertificatePathUrlrequester->setEnabled(true);
 		dlg.main->RacoonCertificatePathUrlrequester->setURL( "/etc/openvpn" );
-		dlg.main->FreeswanGroupBox->setEnabled( false );
+		dlg.main->FreeswanGroupBox->setEnabled( true );
+// 		dlg.main->FreeswanGroupBox->setCurrentText(i18n("OpenVPN"));
 		dlg.main->PrivateKeyPasswordEdit->setEnabled( true );
 		dlg.main->PrivateKeyAgainPasswordEdit->setEnabled( true );
 
 		dlg.main->FilenameUrlrequester->setURL( p12file );
 		dlg.main->ImportPasswordEdit->setEnabled( true );
 		dlg.main->P12GroupBox->setEnabled( true );
-		dlg.main->FreeswanGroupBox->setEnabled( false );
+		dlg.main->FreeswanGroupBox->setEnabled( true );
 		dlg.main->FilenameUrlrequester->setFilter( "*.p12" );
 
 		dlg.main->ImportPasswordEdit->setFocus();
