@@ -25,11 +25,13 @@
 #include <kmessagebox.h>
 #include <klocale.h>
 #include <iostream>
+#include <kstandarddirs.h>
 //END INCLUDES
 
-KVpncConfig::KVpncConfig(QWidget *parent)
+KVpncConfig::KVpncConfig(QWidget *parent,KStatusBar *statusbar)
 {
 	this->parent = parent;
+	this->statusbar = statusbar;
 
 	pathToVpnc = "";
 	pathToRacoon = "";
@@ -46,6 +48,8 @@ KVpncConfig::KVpncConfig(QWidget *parent)
 	pathToIfconfig="/sbin/ifconfig";
 	pathToRoute="/sbin/route";
 	pathToNetstat="/bin/netstat";
+	pathToL2tpd="/usr/sbin/l2tpd";
+	pathToPkcs11Tool="/usr/bin/pkcs11-tool";
 	AutoConnectProfile="";
 	Vpnc_pid_file = "";
 	Pppd_pid_file = "";
@@ -94,6 +98,9 @@ KVpncConfig::KVpncConfig(QWidget *parent)
 	skipKwalletStoring = false;
 	hideOnCloseInfo = false;
 	enableFreeswanVerbose=false;
+	doKillL2tpdIfStillRunning=true;
+	doKillRacoonIfStillRunning=true;
+	hideMainWindow=false;
 
 	VpncDebugLevel = 0;
 	RacoonDebugLevel = "info";
@@ -101,6 +108,8 @@ KVpncConfig::KVpncConfig(QWidget *parent)
 	OpenvpnDebugLevel =0;
 	LogViewerFontSize=0; // can be from -3 up to +3
 
+	DebugBackgroundcolor=black;
+
 	ToolList = new QPtrList<ToolInfo>();
 	ToolList->setAutoDelete( TRUE );
 
@@ -156,8 +165,7 @@ void KVpncConfig::appendLogEntry( QString msg, LogType type )
 				logfile.flush();
 			}
 		}
-
-		if (logfile.isOpen())
+		else
 		{
 			QString date = QDate().currentDate().toString(Qt::ISODate)+ " ";
 			QString time = QTime().currentTime().toString(Qt::ISODate)+ " ";
@@ -180,10 +188,10 @@ void KVpncConfig::appendLogEntry( QString msg, LogType type )
 					prefix=i18n("debug")+": ";
 					break;
 			}
-			appPointer->processEvents();
+// 			appPointer->processEvents();
 			logFileStream << date << time << prefix << msg << "\n";
 			logfile.flush();
-			appPointer->processEvents();
+// 			appPointer->processEvents();
 		}
 
 
@@ -248,10 +256,24 @@ void KVpncConfig::appendLogEntry( QString msg, LogType type )
 			postfix+="</font>";
 
 		LogOutput->append( prefix+msg+postfix );
-		appPointer->processEvents();
+// 		appPointer->processEvents();
 	}
 }
 
+void KVpncConfig::slotStatusMsg( const QString &text, int id )
+{
+	if ( id != ID_FLASH_MSG )
+	{
+		statusbar ->clear();
+		statusbar ->changeItem( text, id );
+	}
+	else
+	{
+		statusbar->message( text, 2000 );
+	}
+
+}
+
 void KVpncConfig::saveOptions()
 {
 	KStandardDirs * dirs = KGlobal::dirs();
@@ -261,6 +283,7 @@ void KVpncConfig::saveOptions()
 	config->setGroup( "General Options" );
 
 	config->writeEntry( "Minimize after connect", minimizeAfterConnect );
+	config->writeEntry("Hide on startup", hideMainWindow );
 	config->writeEntry( "Use silent quit", useSilentQuit );
 	config->writeEntry( "Dont quit on CloseEvent", dontQuitOnCloseEvent );
 	config->writeEntry( "Enable pppd debug", enableDebugPppd );
@@ -290,6 +313,7 @@ void KVpncConfig::saveOptions()
 	config->writeEntry( "Path to ifconfig", pathToIfconfig );
 	config->writeEntry( "Path to route", pathToRoute );
 	config->writeEntry( "Path to netstat", pathToNetstat );
+	config->writeEntry( "Path to pkcs11-tool", pathToPkcs11Tool );
 	config->writeEntry( "Show Statusbar", showStatusBar );
 	config->writeEntry( "Try connect timeout", tryConnectTimeout );
 	config->writeEntry( "Last Profile", lastProfile );
@@ -301,6 +325,7 @@ void KVpncConfig::saveOptions()
 	config->writeEntry( "Error logwindow color", ErrorLogColor );
 	config->writeEntry( "Success  logwindow color", SuccessLogColor );
 	config->writeEntry( "Debug logwindow color", DebugLogColor );
+	config->writeEntry("Debug background color", DebugBackgroundcolor );
 	config->writeEntry( "Use colorized log output", useColorizedLogOutput );
 	config->writeEntry( "Show interface IP in tooltip", showInterfaceIPinTooltip );
 	config->writeEntry( "Auto connect profile", AutoConnectProfile );
@@ -314,6 +339,9 @@ void KVpncConfig::saveOptions()
 	config->writeEntry( "hideOnCloseInfo" ,hideOnCloseInfo);
 //   config->writeEntry( "ToolBar Settings" ,parent->toolBar( "mainToolBar" ));
 	config->writeEntry("Use KWallet",useKwallet);
+	config->writeEntry("FeedbackMailSent",feedbackMailSent);
+	config->writeEntry("Do kill l2tpd if still running",doKillL2tpdIfStillRunning);
+	config->writeEntry("Do kill racoon if still running",doKillRacoonIfStillRunning);
 	appPointer->processEvents();
 
 	if (useKwallet && KWallet::Wallet::isEnabled())
@@ -340,7 +368,7 @@ void KVpncConfig::saveOptions()
 				{
 					if (!skipKwalletStoring )
 					{
-						if (KvpncDebugLevel > 0)
+						if (KvpncDebugLevel > 2)
 							appendLogEntry(i18n( "Wallet enabled and available, writing to wallet." ),debug);
 						// Open local wallet
 						wallet = KWallet::Wallet::openWallet(KWallet::Wallet::LocalWallet());
@@ -353,19 +381,20 @@ void KVpncConfig::saveOptions()
 							if (!wallet->hasFolder(walletname))
 							{
 								walletOK = wallet->createFolder(walletname);
+								wallet->sync();
 								appPointer->processEvents();
 							}
 							if (walletOK)
 							{
 								wallet->setFolder(walletname);
-								appPointer->processEvents();
+// 								appPointer->processEvents();
 								//std::cout << "[set] account: " << ProfileName << ", loginname: " << loginname << ", password: " << pwd << std::endl;
 								bool UserPasswordOK=true;
 								if ( it->getSaveUserPassword() )
 									UserPasswordOK = (wallet->writePassword (QString(name+"__user_pw"), it->getUserPassword()) == 0);
 								else
 									UserPasswordOK = (wallet->writePassword (QString(name+"__user_pw"), "") == 0);
-
+								wallet->sync();
 								appPointer->processEvents();
 
 								bool PskOK=true;
@@ -373,14 +402,16 @@ void KVpncConfig::saveOptions()
 									PskOK = (wallet->writePassword (QString(name+"__psk"), it->getPreSharedKey()) == 0);
 								else
 									PskOK = (wallet->writePassword (QString(name+"__psk"), "") == 0);
+								wallet->sync();
 
 								appPointer->processEvents();
 
 								bool PskKeyPassOK = (wallet->writePassword (QString(name+"__priv_key_pwd"), it->getPrivateKeyPass()) == 0);
+								wallet->sync();
 
 								appPointer->processEvents();
 
-								if (KvpncDebugLevel > 0)
+								if (KvpncDebugLevel > 2)
 								{
 									if (UserPasswordOK)
 										appendLogEntry(i18n( "write of %1 was ok." ).arg(i18n("user password")), debug);
@@ -454,7 +485,7 @@ void KVpncConfig::saveOptions()
 					}
 					else //sessionSaving neither writing to wallet nor to config-file
 					{
-					if (KvpncDebugLevel > 0)
+					if (KvpncDebugLevel > 2)
 							appendLogEntry(i18n( "Writing into Wallet is not possible at shutdown, skipping." ),debug);
 					}
 				}
@@ -462,7 +493,7 @@ void KVpncConfig::saveOptions()
 				{
 					//if (!useKwallet)
 					{
-						if (KvpncDebugLevel > 0)
+						if (KvpncDebugLevel > 2)
 							appendLogEntry(i18n( "Wallet disabled or not available, writing to config file." ),debug);
 						// write passwords to config file :|
 						if ( it->getSaveUserPassword() )
@@ -484,8 +515,12 @@ void KVpncConfig::saveOptions()
 					config->writeEntry( "Connection type", "cisco" );
 				else if ( it->getConnectionType() == VpnAccountData::racoon )
 					config->writeEntry( "Connection type", "racoon" );
+				else if ( it->getConnectionType() == VpnAccountData::l2tpd_racoon )
+					config->writeEntry( "Connection type", "l2tpd (racoon)" );
 				else if ( it->getConnectionType() == VpnAccountData::freeswan )
 					config->writeEntry( "Connection type", "freeswan" );
+				else if ( it->getConnectionType() == VpnAccountData::l2tpd_freeswan )
+					config->writeEntry( "Connection type", "l2tpd (openswan)" );
 				else if ( it->getConnectionType() == VpnAccountData::pptp )
 					config->writeEntry( "Connection type", "pptp" );
 				else if ( it->getConnectionType() == VpnAccountData::openvpn )
@@ -498,6 +533,7 @@ void KVpncConfig::saveOptions()
 
 				config->writeEntry( "Save user password", it->getSaveUserPassword() );
 				config->writeEntry( "Save PSK", it->getSavePsk() );
+				config->writeEntry( "Save private key password", it->getSavePrivateKeyPassword() );
 				config->writeEntry( "Private key", it->getPrivateKey() );
 				config->writeEntry( "Use advanced settings", it->getUseAdvancedSettings() );
 				config->writeEntry( "Application version", it->getApplicationVersion() );
@@ -506,6 +542,7 @@ void KVpncConfig::saveOptions()
 				config->writeEntry( "NT domain name", it->getNtDomainName() );
 				config->writeEntry( "Perfect forward secrety", it->getPerfectForwardSecrety() );
 				config->writeEntry( "IKE group", it->getIkeGroup() );
+				
 				config->writeEntry( "Use application version", it->getUseApplicationVersion() );
 				config->writeEntry( "Use global IPSec secret", it->getUseGlobalIpsecSecret() );
 				config->writeEntry( "Use IKE group", it->getUseIkeGroup() );
@@ -571,6 +608,8 @@ void KVpncConfig::saveOptions()
 				config->writeEntry( "PSK is in file", it->getPskIsInFile() );
 				config->writeEntry( "Use additional network routes", it->getUseAdditionalNetworkRoutes() );
 				config->writeEntry( "Hash algorithm", it->getHashAlgo() );
+				config->writeEntry( "Encryption algorithm", it->getEncryptionAlgorithm() );
+				config->writeEntry( "Authentication algorithm", it->getAuthenticationAlgorithm() );
 				config->writeEntry( "Tunnel device type", it->getTunnelDeviceType() );
 				config->writeEntry( "Use userdefined port", it->getUseUserdefinedPort() );
 				config->writeEntry( "Userdefined port", it->getUserdefinedPort() );
@@ -610,13 +649,27 @@ void KVpncConfig::saveOptions()
 				config->writeEntry( "Fragment", it->getFragment() );
 				config->writeEntry( "PeerTimeout", it->getPeerTimeout() );
 				config->writeEntry("Disable CCP", it->getDisableCcp() );
-				config->writeEntry("Use smartcard", it->getUseSmardcard() );
+				config->writeEntry("Use smartcard", it->getUseSmartcard() );
 				config->writeEntry("Use Xauth interactive", it->getUseXauthInteractive() );
 				config->writeEntry("Use HTTP Proxy Authentication", it->getUseHttpProxyAuth() );
 				config->writeEntry("Use HTTP Proxy Authtype", it->getHttpProxyAuthType() );
 				config->writeEntry("Use HTTP Proxy Auth user", it->getHttpProxyUser() );
 				config->writeEntry("Use HTTP Proxy Auth pass", it->getHttpProxyPass() );
 				config->writeEntry("Use only CA cert and user auth", it->getUseOnlyCaCertAndUserAuth() );
+				config->writeEntry("Use mail address as identifier", it->getUseMailAddressAsIdentifier() );
+				config->writeEntry("Use right next hop", it->getUseRightNextHop() );
+				config->writeEntry("Use left next hop", it->getUseLeftNextHop() );
+				config->writeEntry("Pkcs11 providers", it->getPkcs11Providers() );
+				config->writeEntry("Pkcs11 slot type", it->getPkcs11SlotType() );
+				config->writeEntry("Pkcs11 slot", it->getPkcs11Slot() );
+				config->writeEntry("Pkcs11 id type", it->getPkcs11IdType() );
+				config->writeEntry("Pkcs11 id", it->getPkcs11Id() );
+				config->writeEntry("Pkcs11 sign mode", it->getPkcs11SignMode() );
+				config->writeEntry("Use Pkcs11 providers", it->getUsePkcs11Providers() );
+				config->writeEntry("Allow empty private key password", it->getAllowEmptyPrivateKeyPassword());
+				config->writeEntry("Allow ip address change of peer", it->getAllowIpAddressChangeOfPeer());
+				config->writeEntry("Auth method", it->getAuthMethod());
+				config->writeEntry("Command after connect delay time", it->getCommandAfterConnectDelayTime());
 
 				appPointer->processEvents();
 
@@ -634,13 +687,16 @@ void KVpncConfig::saveOptions()
 
 
 				/* give info */
-				//slotStatusMsg ( i18n( "Profile \"%1\" saved." ).arg( ProfileName.remove("Profile_") ), ID_FLASH_MSG );
-				appendLogEntry ( i18n( "Profile \"%1\" saved." ).arg( ProfileName.remove("Profile_") ) , info);
+				slotStatusMsg ( i18n( "Profile \"%1\" saved." ).arg( ProfileName.remove("Profile_") ), ID_FLASH_MSG );
+
+				if (KvpncDebugLevel > 2)
+					appendLogEntry ( i18n( "Profile \"%1\" saved." ).arg( ProfileName.remove("Profile_") ) , info);
 				appPointer->processEvents();
 			}
 		}
 	}
 	config->sync();
+	doBackupConfig();
 	appPointer->processEvents();
 }
 
@@ -651,6 +707,7 @@ void KVpncConfig::loadOptions()
 
 	config->setGroup( "General Options" );
 	minimizeAfterConnect = config->readBoolEntry( "Minimize after connect", true );
+	hideMainWindow = config->readBoolEntry("Hide on startup", false );
 	useSilentQuit = config->readBoolEntry( "Use silent quit", false );
 	dontQuitOnCloseEvent = config->readBoolEntry( "Dont quit on CloseEvent", true );
 	showDebugConsole = config->readBoolEntry( "Show debugconsole", true );
@@ -684,6 +741,7 @@ void KVpncConfig::loadOptions()
 	pathToKillall = config->readEntry ("Path to kill-all","/usr/bin/killall");
 	pathToPing = config->readEntry ("Path to ping","/bin/ping");
 	pathToOpenvpn = config->readEntry ("Path to openvpn","/usr/sbin/openvpn");
+	pathToPkcs11Tool = config->readEntry( "Path to pkcs11-tool", "/usr/bin/pkcs11-tool" );
 
 	tryConnectTimeout = QString( config->readEntry( "Try connect timeout", "10" ) ).toInt();
 	lastProfile = config->readEntry( "Last Profile", "" );
@@ -696,10 +754,11 @@ void KVpncConfig::loadOptions()
 		useDefaultPaths=true;
 
 	InfoLogColor = config->readColorEntry ( "Info logwindow color", new QColor(Qt::black) );
-	RemoteLogColor = config->readColorEntry ( "Remote logwindow color", new QColor(Qt::black) );
+	RemoteLogColor = config->readColorEntry ( "Remote logwindow color", new QColor(Qt::blue) );
 	ErrorLogColor = config->readColorEntry( "Error logwindow color", new QColor(Qt::red ));
 	SuccessLogColor = config->readColorEntry( "Success  logwindow color", new QColor(Qt::darkGreen) );
 	DebugLogColor = config->readColorEntry( "Debug logwindow color", new QColor(255,175,0 ));
+	DebugBackgroundcolor = config->readColorEntry("Debug background color", new QColor (Qt::white) );
 	useColorizedLogOutput = config->readBoolEntry("Use colorized log output", true);
 	showInterfaceIPinTooltip = config->readBoolEntry( "Show interface IP in tooltip", true );
 	AutoConnectProfile= config->readEntry( "Auto connect profile", "" );
@@ -748,8 +807,12 @@ void KVpncConfig::loadOptions()
 
 	useKwallet = config->readBoolEntry("Use KWallet", true);
 	firstUseOfKwallet = config->readBoolEntry( "First use of Kwallet", true );
+	feedbackMailSent = config->readBoolEntry("FeedbackMailSent",false);
+	doKillL2tpdIfStillRunning = config->readBoolEntry("Do kill l2tpd if still running",true);
+	doKillRacoonIfStillRunning = config->readBoolEntry("Do kill racoon if still running",true);
 
-	if ( KvpncDebugLevel > 0 )
+
+	if ( KvpncDebugLevel > 2 )
 		appendLogEntry( i18n( "Global configuration loaded." ) ,info);
 
 
@@ -786,7 +849,7 @@ void KVpncConfig::loadOptions()
 
 			sessionFound = true;
 
-			if ( KvpncDebugLevel > 0 )
+			if ( KvpncDebugLevel > 2 )
 			{
 				QString tmp = i18n( "Profile found: " );
 				tmp += name.right( ( name.length() ) - 8 );
@@ -800,7 +863,7 @@ void KVpncConfig::loadOptions()
 			/* passwords with kwallet */
 			if (useKwallet && KWallet::Wallet::isEnabled() && !firstUseOfKwallet)
 			{
-				if (KvpncDebugLevel > 0)
+				if (KvpncDebugLevel > 2)
 					appendLogEntry(i18n( "Wallet enabled and available, reading passwords from wallet." ),debug);
 				// Open local wallet
 				wallet = KWallet::Wallet::openWallet(KWallet::Wallet::LocalWallet());
@@ -809,29 +872,35 @@ void KVpncConfig::loadOptions()
 					QString walletname="kvpnc";
 
 					if (!wallet->hasFolder(walletname))
+					{
 						wallet->createFolder(walletname);
+						wallet->sync();
+					}
 
 					if (wallet->setFolder(walletname))
 					{
-						if (KvpncDebugLevel > 0)
+						if (KvpncDebugLevel > 2)
 							appendLogEntry(i18n( "Folder for kvpnc has been set." ),debug);
 
 						QString profilename = name.right( ( name.length() ) - 8 );
 
 						if (wallet->readPassword(QString(profilename+"__user_pw"), UserPassword) != 0)
-							appendLogEntry(i18n( "read of %1 has failed." ).arg(i18n("user password")),debug);
+							appendLogEntry(i18n( "read of %1 has failed." ).arg(i18n("user password")),error);
 						else
-							appendLogEntry(i18n( "read of %1 was successful." ).arg(i18n("user password")),debug);
+							if (KvpncDebugLevel > 2)
+								appendLogEntry(i18n( "read of %1 was successful." ).arg(i18n("user password")),debug);
 
 						if (wallet->readPassword(QString(profilename+"__psk"), PreSharedKey) != 0)
-							appendLogEntry(i18n( "read of %1 has failed." ).arg(i18n("preshared key")),debug);
+							appendLogEntry(i18n( "read of %1 has failed." ).arg(i18n("preshared key")),error);
 						else
-							appendLogEntry(i18n( "read of %1 was successful." ).arg(i18n("preshared key")),debug);
+							if (KvpncDebugLevel > 2)
+								appendLogEntry(i18n( "read of %1 was successful." ).arg(i18n("preshared key")),debug);
 
 						if (wallet->readPassword(QString(profilename+"__priv_key_pwd"), PrivateKeyPass) != 0)
-							appendLogEntry(i18n( "read of %1 has failed." ).arg(i18n("preshared key password")),debug);
+							appendLogEntry(i18n( "read of %1 has failed." ).arg(i18n("preshared key password")),error);
 						else
-							appendLogEntry(i18n( "read of %1 was successful." ).arg(i18n("preshared key password")),debug);
+							if (KvpncDebugLevel > 2)
+								appendLogEntry(i18n( "read of %1 was successful." ).arg(i18n("preshared key password")),debug);
 					}
 					else
 					{
@@ -850,7 +919,7 @@ void KVpncConfig::loadOptions()
 			}
 			else
 			{
-				if (KvpncDebugLevel > 0)
+				if (KvpncDebugLevel > 2)
 				{
 					if (useKwallet && KWallet::Wallet::isEnabled() && firstUseOfKwallet)
 						appendLogEntry(i18n( "Wallet enabled, available but first time, reading passwords from config file." ),debug);
@@ -869,8 +938,12 @@ void KVpncConfig::loadOptions()
 				ConnType = VpnAccountData::cisco;
 			else if ( type_string == "racoon" )
 				ConnType = VpnAccountData::racoon;
+			else if ( type_string == "l2tpd (racoon)" )
+				ConnType = VpnAccountData::l2tpd_racoon;
 			else if ( type_string == "freeswan" )
 				ConnType = VpnAccountData::freeswan;
+			else if ( type_string == "l2tpd (openswan)" )
+				ConnType = VpnAccountData::l2tpd_freeswan;
 			else if ( type_string == "pptp" )
 				ConnType = VpnAccountData::pptp;
 			else if ( type_string == "openvpn" )
@@ -886,6 +959,7 @@ void KVpncConfig::loadOptions()
 			data->setUserName( config->readEntry( "Username", "" ));
 			data->setSaveUserPassword( config->readBoolEntry( "Save user password", false ));
 			data->setSavePsk( config->readBoolEntry( "Save PSK", false ));
+			data->setSavePrivateKeyPassword( config->readBoolEntry( "Save private key password", true ) );
 			data->setUseAdvancedSettings( config->readBoolEntry( "Use advanced settings", false ));
 			data->setApplicationVersion( config->readEntry( "Application version", "Cisco Systems VPN Client 0.3-rm+zomb.1:Linux" ));
 			data->setLocalPort( config->readNumEntry( "Local port", 500 ));
@@ -963,6 +1037,8 @@ void KVpncConfig::loadOptions()
 			QStringList AdditionalNetworkRoutes(QStringList::split( ",", AdditionalNetworkRoutesString ));
 			data->setAdditionalNetworkRoutes(AdditionalNetworkRoutes);
 			data->setHashAlgo( config->readEntry( "Hash algorithm", "md5" ));
+			data->setEncryptionAlgorithm( config->readEntry( "Encryption algorithm", "3des" ));
+			data->setAuthenticationAlgorithm( config->readEntry( "Authentication algorithm", "hmac_sha1" ));
 			data->setTunnelDeviceType( config->readEntry( "Tunnel device type", "tun" ));
 			data->setUseUserdefinedPort( config->readBoolEntry( "Use userdefined port", false ));
 			data->setUserdefinedPort( config->readNumEntry( "Userdefined port", 1194 ));
@@ -1007,13 +1083,27 @@ void KVpncConfig::loadOptions()
 			data->setFragment( config->readNumEntry( "Fragment", 1300) );
 			data->setPeerTimeout( config->readNumEntry( "PeerTimeout",0) );
 			data->setDisableCcp( config->readBoolEntry("Disable CCP",false) );
-			data->setUseSmardcard( config->readBoolEntry("Use smartcard", false) );
+			data->setUseSmartcard( config->readBoolEntry("Use smartcard", false) );
 			data->setUseXauthInteractive(config->readBoolEntry("Use Xauth interactive", false) );
 			data->setUseHttpProxyAuth( config->readBoolEntry("Use HTTP Proxy Authentication", false ) );
 			data->setHttpProxyAuthType( config->readEntry("Use HTTP Proxy Authtype", "basic") );
 			data->setHttpProxyUser( config->readEntry("Use HTTP Proxy Auth user", "") );
 			data->setHttpProxyPass( config->readEntry("Use HTTP Proxy Auth pass", "") );
 			data->setUseOnlyCaCertAndUserAuth(config->readBoolEntry("Use only CA cert and user auth", false) );
+			data->setUseMailAddressAsIdentifier(config->readBoolEntry("Use mail address as identifier", false) );
+			data->setUseRightNextHop(config->readBoolEntry("Use right next hop", false) );
+			data->setUseLeftNextHop(config->readBoolEntry("Use left next hop", false) );
+			data->setPkcs11Providers(config->readEntry("Pkcs11 providers", "/usr/local/lib/libetpkcs11.so" ) );
+			data->setPkcs11SlotType(config->readEntry("Pkcs11 slot type", "id") );
+			data->setPkcs11Slot(config->readEntry("Pkcs11 slot", "0") );
+			data->setPkcs11IdType(config->readEntry("Pkcs11 id type", "id") );
+			data->setPkcs11Id(config->readEntry("Pkcs11 id","" ) );
+			data->setPkcs11SignMode(config->readEntry("Pkcs11 sign mode", "auto") );
+			data->setUsePkcs11Providers(config->readBoolEntry("Use Pkcs11 providers", false) );
+			data->setAllowEmptyPrivateKeyPassword( config->readBoolEntry("Allow empty private key password", false ));
+			data->setAllowIpAddressChangeOfPeer(config->readBoolEntry("Allow ip address change of peer", false));
+			data->setAuthMethod(config->readEntry("Auth method", "chap"));
+			data->setCommandAfterConnectDelayTime(config->readNumEntry("Command after connect delay time", 5));
 
 			appPointer->processEvents();
 			AccountList->append( data );
@@ -1050,10 +1140,10 @@ void KVpncConfig::loadOptions()
 
 			// now mark for deleting old configuration
 			oldConfigurationFound = true;
-			if ( KvpncDebugLevel > 0 )
+			if ( KvpncDebugLevel > 2 )
 				appendLogEntry ( i18n( "Old configuration found, converted." ),info );
 
-			if ( KvpncDebugLevel > 0 )
+			if ( KvpncDebugLevel > 2 )
 				appendLogEntry( i18n( "Configuration for profile \"%1\" loaded." ).arg( Name ),info );
 
 			appPointer->processEvents();
@@ -1068,11 +1158,51 @@ void KVpncConfig::loadOptions()
 		QString filePath = dirs->findResource ( "config", "kvpncrc" );
 		QFile::remove
 			( filePath );
-		if ( KvpncDebugLevel > 0 )
+		if ( KvpncDebugLevel > 2 )
 			appendLogEntry ( i18n( "Old configuration deleted." ), debug );
 	}
 }
 
+void KVpncConfig::doBackupConfig()
+{
+	KStandardDirs * dirs = KGlobal::dirs();
+	QString  config = QString(dirs->saveLocation( "config" )+"/kvpncrc");
+	QFile OriginalConfFile( config );
+	QFile backupOriginalConfFile( config+".backup" );
+	QTextStream writestream( &backupOriginalConfFile );
+	QTextStream readstream( &OriginalConfFile );
+	if ( OriginalConfFile.open( IO_ReadOnly ) )
+	{
+		if (backupOriginalConfFile.open(IO_WriteOnly))
+		{
+			QString OriginalConfFileContent = QString( OriginalConfFile.readAll() )	;
+			writestream << OriginalConfFileContent;
+			OriginalConfFile.close();
+			backupOriginalConfFile.close();
+		}
+	}
+}
+
+void KVpncConfig::restoreBackupConfig()
+{
+	KStandardDirs * dirs = KGlobal::dirs();
+	QString  config = QString(dirs->saveLocation( "config" )+"/kvpncrc");
+	QFile OriginalConfFile( config+".backup" );
+	QFile backupOriginalConfFile( config );
+	QTextStream writestream( &backupOriginalConfFile );
+	QTextStream readstream( &OriginalConfFile );
+	if ( OriginalConfFile.open( IO_ReadOnly ) )
+	{
+		if (backupOriginalConfFile.open(IO_WriteOnly))
+		{
+			QString OriginalConfFileContent = QString( OriginalConfFile.readAll() )	;
+			writestream << OriginalConfFileContent;
+			OriginalConfFile.close();
+			backupOriginalConfFile.close();
+		}
+	}
+}
+
 void KVpncConfig::getToolsInfo()
 {
 	ToolInfo *it;
@@ -1081,105 +1211,21 @@ void KVpncConfig::getToolsInfo()
 	{
 		for ( it = ToolList->first(); it; it = ToolList->next() )
 		{
-			//FIXME how it could be better?
-			if (it->Name == "vpnc")
-			{
-				if (!pathToVpnc.isEmpty())
-					it->addSearchPath(pathToVpnc);
+
+				
+
+				if(programsInPath)
+				{
+					it->addSearchPath("/sbin");
+					it->addSearchPath("/usr/local/sbin");
+					it->addSearchPath("/usr/sbin");
+					it->addSearchPath("/bin");
+					it->addSearchPath("/usr/local/bin");
+					it->addSearchPath("/usr/bin");
+				}
 				it->collectToolInfo();
-			}
-			else	if (it->Name == "ipsec")
-			{
-				if (!pathToFreeswan.isEmpty())
-					it->addSearchPath(pathToFreeswan);
-				it->collectToolInfo();
-			}
-			else	if (it->Name == "racoon")
-			{
-				if (!pathToRacoon.isEmpty())
-					it->addSearchPath(pathToRacoon);
-				it->collectToolInfo();
-			}
-			else	if (it->Name == "setkey")
-			{
-				if (!pathToSetkey.isEmpty())
-					it->addSearchPath(pathToSetkey);
-				it->collectToolInfo();
-			}
-			else	if (it->Name == "openvpn")
-			{
-				if (!pathToOpenvpn.isEmpty())
-					it->addSearchPath(pathToOpenvpn);
-				it->collectToolInfo();
-			}
-			else	if (it->Name == "openssl")
-			{
-				if (!pathToOpenssl.isEmpty())
-					it->addSearchPath(pathToOpenssl);
-				it->collectToolInfo();
-			}
-			else	if (it->Name == "pppd")
-			{
-				if (!pathToPppd.isEmpty())
-					it->addSearchPath(pathToPppd);
-				it->collectToolInfo();
-			}
-			else	if (it->Name == "pptp")
-			{
-				if (!pathToPptp.isEmpty())
-					it->addSearchPath(pathToPptp);
-				it->collectToolInfo();
-			}
-			else	if (it->Name == "iptables")
-			{
-				if (!pathToIptables.isEmpty())
-					it->addSearchPath(pathToIptables);
-				it->collectToolInfo();
-			}
-			else	if (it->Name == "kill")
-			{
-				if (!pathToKill.isEmpty())
-					it->addSearchPath(pathToKill);
-				it->collectToolInfo();
-			}
-			else	if (it->Name == "killall")
-			{
-				if (!pathToKillall.isEmpty())
-					it->addSearchPath(pathToKillall);
-				it->collectToolInfo();
-			}
-			else	if (it->Name == "ping")
-			{
-				if (!pathToPing.isEmpty())
-					it->addSearchPath(pathToPing);
-				it->collectToolInfo();
-			}
-			else	if (it->Name == "ip")
-			{
-				if (!pathToIp.isEmpty())
-					it->addSearchPath(pathToIp);
-				it->collectToolInfo();
-			}
-			else	if (it->Name == "ifconfig")
-			{
-				if (!pathToIfconfig.isEmpty())
-					it->addSearchPath(pathToIfconfig);
-				it->collectToolInfo();
-			}
-			else	if (it->Name == "route")
-			{
-				if (!pathToRoute.isEmpty())
-					it->addSearchPath(pathToRoute);
-				it->collectToolInfo();
-			}
-			else
-			{
-// 				std::cout << "Invalid tool!" << std::endl;
-			}
-			appPointer->processEvents();
 		}
 	}
-
 }
 
 void KVpncConfig::removeEntry(QString Name)
@@ -1207,110 +1253,60 @@ void KVpncConfig::removeEntry(QString Name)
 					ProfileName += name;
 					config->setGroup( ProfileName );
 
-					// 				/* passwords with kwallet */
-					// 				if ( useKwallet && KWallet::Wallet::isEnabled())
-					// 				{
-					// 					if (KvpncDebugLevel > 0)
-					// 						appendLogEntry(i18n( "Wallet enabled and available, writing to wallet." ),debug);
-					// 					// Open local wallet
-					// 					wallet = KWallet::Wallet::openWallet(KWallet::Wallet::LocalWallet());
-					// 					if (wallet != 0)
-					// 					{
-					// 						QString walletname="kvpnc";
-					// 						// Check if folder exists, otherwise create it
-					// 						bool walletOK=true;
-					// 						if (!wallet->hasFolder(walletname))
-					// 							walletOK = wallet->createFolder(walletname);
-					// 						if (walletOK)
-					// 						{
-					// 							wallet->setFolder(walletname);
-					// 							//std::cout << "[set] account: " << ProfileName << ", loginname: " << loginname << ", password: " << pwd << std::endl;
-					// 							bool UserPasswordOK=true;
-					// 							if ( it->getSaveUserPassword() )
-					// 								UserPasswordOK = (wallet->writePassword (QString(name+"__user_pw"), it->getUserPassword()) == 0);
-					// 							else
-					// 								UserPasswordOK = (wallet->writePassword (QString(name+"__user_pw"), "") == 0);
-					// 							bool PskOK=true;
-					// 							if ( it->getSavePsk() )
-					// 								PskOK = (wallet->writePassword (QString(name+"__psk"), it->getPreSharedKey()) == 0);
-					// 							else
-					// 								PskOK = (wallet->writePassword (QString(name+"__psk"), "") == 0);
-					//
-					// 							bool PskKeyPassOK = (wallet->writePassword (QString(name+"__priv_key_pwd"), it->getPrivateKeyPass()) == 0);
-					//
-					// 							if (KvpncDebugLevel > 0)
-					// 							{
-					// 								if (UserPasswordOK)
-					// 									appendLogEntry(i18n( "write of %1 was ok." ).arg(i18n("user password")), debug);
-					// 								else
-					// 								{
-					// 									// 									if ( KMessageBox::warningContinueCancel( 0,
-					// 									// 									        i18n( "KVpnc is unable to save %1 password securely in your wallet.<br>"
-					// 									// 									              "Do you want to save the password in the <b>unsafe</b> configuration file instead?" ).arg(i18n("user password")),
-					// 									// 									        i18n( "Unable to store secure %1" ).arg(i18n("user password")),
-					// 									// 									        KGuiItem( i18n( "Store &Unsafe" ), QString::fromLatin1( "unlock" ),QString::fromLatin1( "KWalletFallbackToKConfig" ) ) ) == KMessageBox::Continue )
-					// 									// 									{
-					// 									// 										config->deleteEntry( "User password", it->getUserPassword() );
-					// 									// 									}
-					//
-					// 									appendLogEntry(i18n( "write of %1 has failed." ).arg(i18n("user password")), debug);
-					// 								}
-					//
-					// 								if (PskOK)
-					// 									appendLogEntry(i18n( "write of %1 was successful." ).arg(i18n("preshared key")), debug);
-					// 								else
-					// 								{
-					// 									// 									if ( KMessageBox::warningContinueCancel( 0,
-					// 									// 									        i18n( "KVpnc is unable to save %1 password securely in your wallet.<br>"
-					// 									// 									              "Do you want to save the password in the <b>unsafe</b> configuration file instead?" ).arg(i18n("pre shared key")),
-					// 									// 									        i18n( "Unable to store secure %1" ).arg(i18n("pre shared key")),
-					// 									// 									        KGuiItem( i18n( "Store &Unsafe" ), QString::fromLatin1( "unlock" ),QString::fromLatin1( "KWalletFallbackToKConfig" ) ) ) == KMessageBox::Continue )
-					// 									// 									{
-					// 									// 										config->deleteEntry( "Pre shared key", it->getPreSharedKey() );
-					// 									// 									}
-					// 									appendLogEntry(i18n( "write of %1 has failed." ).arg(i18n("preshared key")),debug);
-					// 								}
-					//
-					// 								if (PskOK)
-					// 									appendLogEntry(i18n( "write of %1 was successful." ).arg(i18n("preshared key")), debug);
-					// 								else
-					// 								{
-					// 									// 									if ( KMessageBox::warningContinueCancel( 0,
-					// 									// 									        i18n( "KVpnc is unable to save %1 password securely in your wallet.<br>"
-					// 									// 									              "Do you want to save the password in the <b>unsafe</b> configuration file instead?" ).arg(i18n("preshared key password")),
-					// 									// 									        i18n( "Unable to store secure %1" ).arg(i18n("reshared key password")),
-					// 									// 									        KGuiItem( i18n( "Store &Unsafe" ), QString::fromLatin1( "unlock" ),QString::fromLatin1( "KWalletFallbackToKConfig" ) ) ) == KMessageBox::Continue )
-					// 									// 									{
-					// 									// 										config->deleteEntry( "Pre shared key", it->getPreSharedKey() );
-					// 									// 									}
-					// 									appendLogEntry(i18n( "write of %1 has failed." ).arg(i18n("preshared key")),debug);
-					// 								}
-					// 								if (PskKeyPassOK)
-					// 									appendLogEntry(i18n( "write of %1 was successful." ).arg(i18n("preshared key password")), debug);
-					// 								else
-					// 								{
-					// 									// 									if ( KMessageBox::warningContinueCancel( 0,
-					// 									// 									        i18n( "KVpnc is unable to save %1 password securely in your wallet.<br>"
-					// 									// 									              "Do you want to save the password in the <b>unsafe</b> configuration file instead?" ).arg(i18n("preshared key password")),
-					// 									// 									        i18n( "Unable to store secure %1" ).arg(i18n("reshared key password")),
-					// 									// 									        KGuiItem( i18n( "Store &Unsafe" ), QString::fromLatin1( "unlock" ),QString::fromLatin1( "KWalletFallbackToKConfig" ) ) ) == KMessageBox::Continue )
-					// 									// 									{
-					// 									// 										config->deleteEntry( "Pre shared key", it->getPreSharedKey() );
-					// 									// 									}
-					// 									appendLogEntry(i18n( "write of %1 has failed." ).arg(i18n("preshared key password")),debug);
-					// 								}
-					// 								//std::cout << "success: " << pwdOK << std::endl;
-					// 							}
-					// 						}
-					// 						else
-					// 						{
-					// 					KMessageBox::error( 0, i18n( "Unable to create wallet folder for kvpnc!" ) );
-					// 					appendLogEntry(i18n( "Unable to create wallet folder for kvpnc!" ), error);
-					// 											}
-					// 				}
-					// 			}
-					// 			else
-					// 			{
+					/* passwords with kwallet */
+					if ( useKwallet && KWallet::Wallet::isEnabled())
+					{
+	// 					if (KvpncDebugLevel > 0)
+	// 						appendLogEntry(i18n( "Wallet enabled and available, writing to wallet." ),debug);
+	// 					// Open local wallet
+						wallet = KWallet::Wallet::openWallet(KWallet::Wallet::LocalWallet());
+						if (wallet != 0)
+						{
+							QString walletname="kvpnc";
+							// Check if folder exists, otherwise create it
+							bool walletOK=true;
+							if (!wallet->hasFolder(walletname))
+							{
+								walletOK = wallet->createFolder(walletname);
+								wallet->sync();
+							}
+							if (walletOK)
+							{
+								wallet->setFolder(walletname);
+								//std::cout << "[set] account: " << ProfileName << ", loginname: " << loginname << ", password: " << pwd << std::endl;
+								bool UserPasswordOK = (wallet->removeEntry (QString(name+"__user_pw")) == 0);
+								bool PskOK= (wallet->removeEntry (QString(name+"__psk")) == 0);
+								bool PskKeyPassOK = (wallet->removeEntry (QString(name+"__priv_key_pwd")) == 0);
+								wallet->sync();
+
+								if (KvpncDebugLevel > 2)
+								{
+									if (UserPasswordOK)
+										appendLogEntry(i18n( "delete of %1 was ok." ).arg(i18n("user password")), debug);
+									else
+										appendLogEntry(i18n( "delete of %1 has failed." ).arg(i18n("user password")), debug);
+	//
+									if (PskOK)
+										appendLogEntry(i18n( "delete of %1 was successful." ).arg(i18n("preshared key")), debug);
+									else
+										appendLogEntry(i18n( "delete of %1 has failed." ).arg(i18n("preshared key")),debug);
+
+									if (PskKeyPassOK)
+										appendLogEntry(i18n( "delete of %1 was successful." ).arg(i18n("private key password")), debug);
+									else
+										appendLogEntry(i18n( "delete of %1 has failed." ).arg(i18n("private key password")),debug);
+
+
+	// 								//std::cout << "success: " << pwdOK << std::endl;
+								}
+							}
+							else
+							{
+								KMessageBox::error( 0, i18n( "Unable to create wallet folder for kvpnc!" ) );
+								appendLogEntry(i18n( "Unable to create wallet folder for kvpnc!" ), error);
+						}
+					}
+				}
 					if (KvpncDebugLevel > 0)
 						appendLogEntry(i18n( "Wallet disabled or not available, writing to config file." ),debug);
 					// write passwords to config file :|
@@ -1395,10 +1391,11 @@ void KVpncConfig::removeEntry(QString Name)
 					config->deleteEntry( "Use userdefined port" );
 					config->deleteEntry( "Userdefined port" );
 					config->deleteEntry( "Keep default route" );
-
 					config->deleteEntry( "Additional network routes" );
 					config->deleteEntry("AuthWithUsernameAndPassword" );
 					config->deleteEntry("AllowEmptyGroupPassword" );
+					config->deleteEntry("Encryption algorithm");
+					config->deleteEntry("Authentication algorithm");
 
 					config->deleteGroup( ProfileName );
 
@@ -1416,4 +1413,5 @@ void KVpncConfig::removeEntry(QString Name)
 	appPointer->processEvents();
 }
 
+
 #include "kvpncconfig.moc"
