@@ -655,70 +655,39 @@ void NewProfileDialog::advancedSettingsClicked()
 
 	AdvancedSettingsDialog d1(this, i18n("Advanced Settings"),profileData,connectionType,GlobalConfig);
 
-	d1.main->UseGlobalIpsecSecretCheckbox->setChecked ( profileData->getUseGlobalIpsecSecret() );
 	d1.main->SingleDesCheckbox->setChecked( profileData->getUseSpecialRemoteID() );
 	d1.main->IkeGroupCheckbox->setChecked ( profileData->getUseIkeGroup() );
 
 	d1.main->PerfectForwardSecurityCheckbox->setChecked( profileData->getUsePerfectForwardSecrety() );
-	d1.main->NtDomainNameCheckbox->setEnabled ( profileData->getUseNtDomainName() );
 	d1.main->ApplicationVersionCheckbox->setEnabled ( profileData->getUseApplicationVersion() );
-	d1.main->UseGlobalIpsecSecretCheckbox->setEnabled ( profileData->getUseGlobalIpsecSecret() );
 
-	d1.main->LocalPortSpinbox->setValue( profileData->getLocalPort() );
 	d1.main->IkeGroupCombobox->setCurrentText( profileData->getIkeGroup() );
 	d1.main->PerfectForwardSecrecyCombobox->setCurrentText( profileData->getPerfectForwardSecrety() );
-	d1.main->NtDomainNameLineedit->setText( profileData->getNtDomainName() );
 	d1.main->ApplicationVersionLineedit->setText( profileData->getApplicationVersion() );
 
-	d1.main->NtDomainNameCheckbox->setChecked( profileData->getUseNtDomainName() );
-	d1.main->NtDomainNameLineedit->setText( profileData->getNtDomainName() );
 	d1.main->AdvancedSettingsCheckbox->setChecked ( profileData->getUseAdvancedSettings());
 
 	if ( profileData->getUseAdvancedSettings() )
 	{
 		if (profileData->connectiontype == VpnAccountData::cisco)
 		{
-			d1.main->LocalPortCheckbox->setEnabled ( true );
 			d1.main->PerfectForwardSecurityCheckbox->setEnabled ( true );
 			d1.main->SingleDesCheckbox->setEnabled ( true );
-			d1.main->NtDomainNameCheckbox->setEnabled ( true );
 			d1.main->ApplicationVersionCheckbox->setEnabled ( true );
-			d1.main->UseGlobalIpsecSecretCheckbox->setEnabled ( true );
-		}
-
-		else if (profileData->connectiontype == VpnAccountData::pptp)
-		{
-			d1.main->NtDomainNameCheckbox->setEnabled( true );
 		}
 
 
 		d1.main->IkeGroupCheckbox->setChecked ( profileData->getUseIkeGroup() );
-
 		d1.main->SingleDesCheckbox->setChecked( profileData->getUseSingleDes() );
-
 		d1.main->IkeGroupCheckbox->setChecked ( profileData->getUseIkeGroup() );
-
-		d1.main->UseGlobalIpsecSecretCheckbox->setChecked ( profileData->getUseGlobalIpsecSecret() );
-
 		d1.main->PerfectForwardSecurityCheckbox->setChecked( profileData->getUsePerfectForwardSecrety() );
 
-		if ( profileData->getUseNtDomainName() )
-		{
-			d1.main->NtDomainNameLineedit->setEnabled ( true );
-			d1.main->NtDomainNameCheckbox->setChecked( true );
-		}
-
 		if ( profileData->getUseApplicationVersion() )
 		{
 			d1.main->ApplicationVersionLineedit->setEnabled ( true );
 			d1.main->ApplicationVersionCheckbox->setChecked( true );
 		}
 
-		if ( profileData->getUseLocalPort() )
-		{
-			d1.main->LocalPortSpinbox->setEnabled ( true );
-			d1.main->LocalPortCheckbox->setChecked( true );
-		}
 
 		d1.main->IkeGroupCombobox->setEnabled (  profileData->getUseIkeGroup() );
 	}
