@@ -25,6 +25,7 @@
 #include <qptrlist.h>
 #include <qtextedit.h>
 #include <qcolor.h>
+#include <qscrollview.h>
 
 #include "vpnaccountdata.h"
 
@@ -35,14 +36,18 @@
 #include "configlogoptions.h"
 #include "configdaemonoptions.h"
 #include "confighelperprogramoptions.h"
-#include "profilecertpskoptions.h"
+#include "profilecertoptions.h"
+#include "profilesmartcardoptions.h"
+#include "profilepskoptions.h"
 #include "profileciscooptions.h"
 #include "profilecmdexecafterconnectoptions.h"
 #include "profilecmdexecafterdisconnectoptions.h"
 #include "profilecmdexecbeforeconnectoptions.h"
 #include "profilecmdexecbeforedisconnectoptions.h"
 #include "profilegeneraloptions.h"
+#include "profilenetworkgeneraloptions.h"
 #include "profilenetworknatoptions.h"
+#include "profilenetworkhttpproxyoptions.h"
 #include "profilenetworkrouteoptions.h"
 #include "profilenetworkvirtualipoptions.h"
 #include "profileopenvpnoptions.h"
@@ -85,6 +90,7 @@ public slots:
 	void pageChanged(QWidget *);
 	void userAuthToggled(bool);
 	void useUdpToggled(bool);
+	void useSpecialServerSmartcardificateToggled(bool);
 
 private slots:
 	void OpenvpnPathChanged(const QString&);
@@ -92,11 +98,16 @@ private slots:
 	void PppdPathChanged(const QString&);
 	void PptpPathChanged(const QString&);
 	void RacoonPathChanged(const QString&);
-	void SetkeyPathChanged(const QString&);
 	void VpncPathChanged(const QString&);
+	void L2tpdPathChanged(const QString&);
 	
 	void lineeditTextChanged(const QString &);
 	void checkboxStateChanged(int);
+	void showEvent(QShowEvent* e);
+	void detectPkcs11Ids();
+	void detectPkcs11Slots();
+	void localPortToggled(bool);
+
 
 protected:
 	void setupGui();
@@ -109,6 +120,7 @@ protected:
 	bool canAccept;
 	QPixmap ok_icon;
 	QPixmap wrong_icon;
+	QScrollView* DaemonScrollView;
 
 public:
 	QApplication *app;
@@ -127,11 +139,15 @@ public:
 	// profile
 	ProfileGeneralOptions *ProfileGeneralOptionsWidget;
 	ProfileNetworkRouteOptions *ProfileNetworkRouteOptionsWidget;
+	ProfileNetworkGeneralOptions *ProfileNetworkGeneralOptionsWidget;
 	ProfileNetworkNatOptions *ProfileNetworkNatOptionsWidget;
 	ProfileNetworkVirtualIpOptions *ProfileNetworkVirtualIpOptionsWidget;
-	ProfileCertPskOptions *ProfileCertPskOptionsWidget;
+	ProfileCertOptions *ProfileCertOptionsWidget;
+	ProfileSmartcardOptions *ProfileSmartcardOptionsWidget;
+	ProfilePskOptions *ProfilePskOptionsWidget;
 	ProfileUserOptions *ProfileUserOptionsWidget;
 	ProfileOpenvpnOptions *ProfileOpenvpnOptionsWidget;
+	ProfileNetworkHttpProxyOptions *ProfileNetworkHttpProxyOptionsWidget;
 	ProfilePptpOptions *ProfilePptpOptionsWidget;
 	ProfileRacoonOptions *ProfileRacoonOptionsWidget;
 	ProfileCiscoOptions *ProfileCiscoOptionsWidget;
@@ -143,11 +159,15 @@ public:
 
 	QComboBox *ProfileGeneralSelector;
 	QComboBox *ProfileNetworkRouteSelector;
+	QComboBox *ProfileNetworkGeneralSelector;
 	QComboBox *ProfileNetworkNatSelector;
 	QComboBox *ProfileNetworkVirtualIpSelector;
-	QComboBox *ProfileCertPskSelector;
+	QComboBox *ProfileCertSelector;
+	QComboBox *ProfileSmartcardSelector;
+	QComboBox *ProfilePskSelector;
 	QComboBox *ProfileUserSelector;
 	QComboBox *ProfileOpenvpnSelector;
+	QComboBox *ProfileNetworkHttpProxySelector;
 	QComboBox *ProfilePptpSelector;
 	QComboBox *ProfileRacoonSelector;
 	QComboBox *ProfileCiscoSelector;
@@ -156,6 +176,8 @@ public:
 	QComboBox *ProfileCmdExecAfterConnectSelector;
 	QComboBox *ProfileCmdExecAfterDisconnectSelector;
 
+	QFrame* DaemonFrame;
+
 	bool profileAddedOrDeleted;
 	bool showOnlyProfiles;
 	QString NewProfileName;
