@@ -24,29 +24,22 @@
 #include <klocale.h>
 #include <kstandarddirs.h>
 #include <kmessagebox.h>
+#include <qtimer.h>
 #include <qstringlist.h>
 
 #include "toolinfo.moc"
 
-ToolInfo::ToolInfo(bool programsInPath, const QString &Name)
+ToolInfo::ToolInfo( const QString &Name)
 {
 	this->Name = Name;
 	this->programsInPath = programsInPath;
 	this->found = false;
+	TryPath_first="";
 	/*
 		if (!PathToExec.isEmpty())
 			SearchPathList.append(PathToExec);*/
-	// 	if (GlobalConfig->KvpncDebugLevel > 0)
-	// 	std::cout << "Checking for: " << Name.ascii() << std::endl;
-
-	// 	SearchPathList.append("/bin");
-	// 	SearchPathList.append("/sbin");
-	// 	SearchPathList.append("/usr/bin");
-	// 	SearchPathList.append("/usr/sbin");
-	// 	SearchPathList.append("/usr/local/bin");
-	// 	SearchPathList.append("/usr/local/sbin");
-	// 	SearchPathList.append("/root/bin");
-	// 	SearchPathList.append(QString (QString(getenv("$HOME"))+"/bin"));
+// 	if (config->KvpncDebugLevel > 2)
+// 		std::cout << "Checking for: " << Name.ascii() << std::endl;
 
 	Version=i18n("No info");
 
@@ -57,27 +50,44 @@ ToolInfo::ToolInfo(bool programsInPath, const QString &Name)
 
 bool ToolInfo::collectToolInfo()
 {
+	if(programsInPath)
+	{
+		addSearchPath("/sbin");
+		addSearchPath("/usr/local/sbin");
+		addSearchPath("/usr/sbin");
+		addSearchPath("/bin");
+		addSearchPath("/usr/local/bin");
+		addSearchPath("/usr/bin");
+		addSearchPath(QString (QString(getenv("$HOME"))+"/bin"));
+	}
+
 	bool success=false;
 
-	QStringList::Iterator it = SearchPathList.begin();
-	QString TryPath_first = *it;
-// 	std::cout << i18n("Trying first %1 at: %2").arg(Name).arg(TryPath_first) << std::endl;
-	if (!TryPath_first.isEmpty()  && QFile (TryPath_first).exists() && QFileInfo(TryPath_first).isFile() )
+	QStringList::Iterator it;
+	if (QFile (TryPath_first).exists() && QFileInfo(TryPath_first).isFile() )
 		{
-// 			std::cout << i18n("%1 found at: %2").arg(Name).arg(TryPath_first) << std::endl;
+// 			if (GlobalConfig->KvpncDebugLevel > 2)
+// 				std::cout << Name.ascii() <<  "found at: " << TryPath_first.ascii() << ", was first try" << std::endl;
 			found=true;
 			PathToExec=TryPath_first;
 	}
 	else
 	{
+// 		if (GlobalConfig->KvpncDebugLevel > 2)
+// 		{
+// 			std::cout << QString(i18n("Trying first %1 at %2 has been failed, trying other paths").arg(Name).arg(TryPath_first)).ascii() << std::endl;
+// 			std::cout << "searchpath list: " << SearchPathList.join(" ").ascii() << std::endl;
+// 		}
 		for (QStringList::Iterator it = SearchPathList.begin();it != SearchPathList.end();it++)
 		{
 			QString TryPath=QString(*it)+"/"+Name;
-// 			std::cout << i18n("Trying %1 at: %2").arg(Name).arg(TryPath) << std::endl;
+// 			if (GlobalConfig->KvpncDebugLevel > 2)
+// 				std::cout << QString (i18n("Trying %1 at: %2").arg(Name).arg(TryPath)).ascii() << std::endl;
 	
 			if (!TryPath.isEmpty()  && QFile (TryPath).exists() && QFileInfo(TryPath).isFile() )
 			{
-// 				std::cout << i18n("%1 found at: %2").arg(Name).arg(TryPath) << std::endl;
+// 				if (GlobalConfig->KvpncDebugLevel > 2)
+// 					std::cout << QString(i18n("%1 found at: %2").arg(Name).arg(TryPath)).ascii() << std::endl;
 				found=true;
 				PathToExec=TryPath;
 				break;
@@ -85,55 +95,7 @@ bool ToolInfo::collectToolInfo()
 		}
 	}
 
-
-	if (PathToExec.isEmpty())
-	{
-		if (programsInPath)
-		{
-			// 			std::cerr << i18n("%1 not found in PATH").arg(Name) << std::endl;
-			PathToExec=KStandardDirs::findExe(Name);
-
-			if (PathToExec.isEmpty() )
-				PathToExec=KStandardDirs::findExe(Name, QString(QString(getenv("$HOME"))+"/bin"));
-			else return false;
-
-			if (PathToExec.isEmpty())
-				PathToExec=KStandardDirs::findExe(Name, QString("/sbin"));
-			else return false;
-
-			if (PathToExec.isEmpty())
-				PathToExec=KStandardDirs::findExe(Name, QString("/usr/sbin"));
-			else return false;
-
-			if (PathToExec.isEmpty())
-				PathToExec=KStandardDirs::findExe(Name, QString("/usr/sbin"));
-			else return false;
-
-			if (PathToExec.isEmpty())
-				PathToExec=KStandardDirs::findExe(Name, QString("/bin"));
-			else return false;
-
-			if (PathToExec.isEmpty())
-				PathToExec=KStandardDirs::findExe(Name, QString("/usr/bin"));
-			else return false;
-
-			if (PathToExec.isEmpty())
-				PathToExec=KStandardDirs::findExe(Name, QString("/usr/local/sbin"));
-			else return false;
-
-			if (PathToExec.isEmpty())
-				PathToExec=KStandardDirs::findExe(Name, QString("/usr/local/bin"));
-			else return false;
-		}
-		else
-		{
-
-		}
-	}
-
-
-
-	if ( found==true)
+	if ( found)
 	{
 
 		CollectToolInfoProcess = new QProcess(this);
@@ -186,10 +148,13 @@ bool ToolInfo::collectToolInfo()
 		else	if (Name == "pptp")
 			;
 		else	if (Name == "l2tpd")
-			;
+			CollectToolInfoProcess->addArgument("-D");
+		else if (Name =="pkcs11-tool")
+			CollectToolInfoProcess->addArgument("-I");
 		else
 		{
-			// 			std::cout << "Invalid tool (" << Name.ascii() <<")!" << std::endl;
+// 			if (GlobalConfig->KvpncDebugLevel > 2)
+// 				std::cout << "Invalid tool (" << Name.ascii() <<")!" << std::endl;
 			run=false;
 			success=false;
 		}
@@ -201,8 +166,7 @@ bool ToolInfo::collectToolInfo()
 
 			if ( !PathToExec.isEmpty() && !CollectToolInfoProcess->start() )
 			{
-				//std::cerr << i18n("Unable to start collectToolInfo process (%1)!").arg(Name) << std::endl;
-
+				std::cerr << i18n("Unable to start collectToolInfo process (%1)!").arg(Name) << std::endl;
 				//delete CollectToolInfoProcess;
 				collectRunning=false;
 				success=false;
@@ -210,8 +174,15 @@ bool ToolInfo::collectToolInfo()
 			else
 			{
 				collectRunning=true;
+
 				while(CollectToolInfoProcess->isRunning() && collectRunning)
-				{}
+				{
+					if (Name == "l2tpd" || Name == "pkcs11-tool") 
+					{
+						usleep(500);
+						CollectToolInfoProcess->kill();
+					}
+				}
 
 				//KMessageBox::information(0,QString("Name: "+Name+", Path: "+PathToExec+", Version: "+Version),"aaa");
 				// 		disconnect( CollectToolInfoProcess, SIGNAL( readyReadStdout() ), this, SLOT( readStdOutCollectToolInfo() ) );
@@ -230,6 +201,11 @@ bool ToolInfo::collectToolInfo()
 		if (Name=="openvpn")
 			getOpenvpnCapabilities();
 	}
+	else
+	{
+// 		if (GlobalConfig->KvpncDebugLevel > 2)
+// 			std::cout << QString(i18n("%1 not found.").arg(Name)).ascii() << std::endl;
+	}
 	return success;
 }
 
@@ -247,8 +223,8 @@ void ToolInfo::readStdOutCollectToolInfo()
 		{
 			if ( msg.find( "version", 0, FALSE ) > -1 )
 			{
-				// 								std::cout << "stdout collectToolInfo: " << msg.ascii() << std::endl;
-				// 								std::cout << "stdout collectToolInfo: 2,2 " << msg.simplifyWhiteSpace().section(' ',2,2).ascii() << std::endl;
+// 												std::cout << "stdout collectToolInfo: " << msg.ascii() << std::endl;
+// 												std::cout << "stdout collectToolInfo: 2,2 " << msg.simplifyWhiteSpace().section(' ',2,2).ascii() << std::endl;
 				Version = msg.simplifyWhiteSpace().section(' ',2,2);
 			}
 		}
@@ -257,10 +233,14 @@ void ToolInfo::readStdOutCollectToolInfo()
 			if ( msg.find( "wan", 0, FALSE ) > -1 )
 			{
 				// 				std::cout << "stdout collectToolInfo: " << msg << std::endl;
-				if ( msg.find( "openswan", 0, FALSE ) > -1 )
-					Version = msg.section(' ',2,2);
-				else
-					Version = msg.section(' ',1,2);
+// 				if ( msg.find( "openswan", 0, FALSE ) > -1 )
+// 					Version = msg.section(' ',2,2);
+// 				else
+					Version = msg.section(' ',1,2); // "Openswan Ux.x.x/kx.x.x"
+					if (msg.section(' ',1,2).contains('/'))
+						Version = msg.section(' ',1,2).section('/',0,0); // "Openswan Ux.x.x"
+					else
+						Version = msg.section(' ',1,2);
 			}
 		}
 		else	if (Name == "racoon")
@@ -339,9 +319,22 @@ void ToolInfo::readStdOutCollectToolInfo()
 				Version = msg.section(' ',3,3).section(')',0,0);
 			}
 		}
+		else	if (Name == "pkcs11-tool")
+		{
+			if ( msg.find( "version", 0, FALSE ) > -1 )
+			{
+				//std::cout << "stdout collectToolInfo: " << msg << std::endl;
+				Version = msg.section(' ',2,2);
+			}
+			else if ( msg.find( "Error: can't open /var/run/openct/status", 0, FALSE ) > -1 )
+			{
+				std::cerr << "stdout collectToolInfo (pkcs11): " << msg.ascii() << std::endl;
+				CollectToolInfoProcess->kill();
+			}
+		}
 		else
 		{
-			;
+			KMessageBox::information(0,QString("Name: "+Name+", Path: "+PathToExec+", Version: "+Version+", err: "+msg),"aaa");
 		}
 	}
 	collectRunning=false;
@@ -352,8 +345,11 @@ void ToolInfo::readStdErrCollectToolInfo()
 	while (CollectToolInfoProcess->canReadLineStderr())
 	{
 		QString msg = QString( CollectToolInfoProcess->readLineStderr() );
-		// 				std::cout << "Checking for: " << Name.ascii() << std::endl;
-		// 		std::cout << "stderr collectToolInfo: " << msg.ascii() <<std::endl;
+// 		if (GlobalConfig->KvpncDebugLevel > 2)
+// 		{
+// 			std::cout << "Checking for: " << Name.ascii() << std::endl;
+// 			std::cout << "stderr collectToolInfo: " << msg.ascii() <<std::endl;
+// 		}
 
 		if (Name == "pppd")
 		{
@@ -396,6 +392,31 @@ void ToolInfo::readStdErrCollectToolInfo()
 				Version = msg.section(' ',2,2).section(')',0,0);
 			}
 		}
+		else	if (Name == "l2tpd")
+		{
+			if ( msg.find( "l2tpd", 0, FALSE ) > -1 )
+			{
+				//std::cout << "stdout collectToolInfo: " << msg << std::endl;
+				Version = msg.section(' ',2,2).section(')',0,0);
+			}
+		}
+		else	if (Name == "pkcs11-tool")
+		{
+			if ( msg.find( "version", 0, FALSE ) > -1 )
+			{
+// 				std::cout << "stdout collectToolInfo (pkcs11): " << msg.ascii() << std::endl;
+				Version = msg.section(' ',2,2);
+			}
+			else if ( msg.find( "Error: can't open /var/run/openct/status", 0, FALSE ) > -1 )
+			{
+// 				std::cout << "stdout collectToolInfo (pkcs11): " << msg.ascii() << std::endl;
+				CollectToolInfoProcess->kill();
+			}
+		}
+		else
+		{
+// 			KMessageBox::information(0,QString("Name: "+Name+", Path: "+PathToExec+", Version: "+Version+", err: "+msg),"aaa");
+		}
 	}
 }
 
@@ -473,7 +494,7 @@ void ToolInfo::getOpenvpnCapabilities()
 
 		if ( !PathToExec.isEmpty() && !CollectOpenvpnCapabilitiesProcess->start() )
 		{
-			//std::cerr << i18n("Unable to start collectToolInfo process (%1)!").arg(Name) << std::endl;
+			std::cerr << i18n("Unable to start collectToolInfo process (%1)!").arg(Name) << std::endl;
 
 			//delete CollectToolInfoProcess;
 			collectOpenvpnCapabilitiesRunning=false;
