@@ -36,7 +36,7 @@ public:
 	 * @param Name Name of the tool
 	 * @return None
 	 */
-	ToolInfo(bool programsInPath, const QString &Name);
+	ToolInfo(const QString &Name);
 
 private:
 	QProcess *CollectToolInfoProcess;
@@ -44,14 +44,15 @@ private:
 	QStringList SearchPathList;
 	bool collectRunning;
 	bool collectOpenvpnCapabilitiesRunning;
-	bool programsInPath;
 
 public:
 	QString Name; //< Name of the tool
 	QString Version; //< Version number of the tool
 	QString PathToExec; //< Path to executable
 	QString Capabilities;
+	QString TryPath_first; //< try that path first before list
 	bool collectToolInfo(); //< Collects information of the tool
+	bool programsInPath;
 	bool found;
 	void addSearchPath(const QString &path); //< Adds a path for searching
 	void removeSearchPath(const QString &path); //< Removes a path for searching
