@@ -52,11 +52,17 @@ public:
 	bool loadKernelModule(QString Name,QApplication *app);
 	bool doChmod(QString file, QString mode);
 	QStringList getOpenvpnCiphers();
+	QStringList getOpenvpnDigests();
+	QStringList getKernelCrypto();
 	QPtrList<ToolInfo>* getToolList();
 	ToolInfo* getToolInfo(KVpncConfig *GlobalConfig, QString name);
 	QString resolveName(QString Name);
 	QString removeSpecialCharsForFilename(QString filename);
 	QString getNameAndPidOfProgramListen(int port);
+	QString getEmailAddressOfCert(QString cert);
+	QStringList getSmartcardSlots(QString ProviderLib);
+	QStringList getSmartcardCertsFromSlot(QString slot,QString IdType,QString ProviderLib);
+	bool getNeedsPassphrase(QString key);
 
 public slots:
 	void readStdOutCreateTunDev();
@@ -67,21 +73,36 @@ public slots:
 	void readStdErrToolsTest();
 	void readStdOutRetriveOpenvpnCiphers();
 	void readStdErrRetriveOpenvpnCiphers();
+	void readStdOutRetriveOpenvpnDigests();
+	void readStdErrRetriveOpenvpnDigests();
 	void readStdOutGetNameAndPidOfProgramListen();
 	void readStdErrGetNameAndPidOfProgramListen();
+	void readOutGetEmailAddressOfCert();
+	void readOutGetSmartcardSlots();
+	void readOutGetSmartcardCertsFromSlot();
+	void readOutNeedsPassphrase();
+	
 
 private:
 	bool modprobeSuccess;
 	bool toolsTestSuccess;
 	bool retrieveOpenvpnCiphers;
+	bool retrieveOpenvpnDigests;
 	bool retrieveNameAndPidOfProgramListen;
+	bool needsPassphrase;
 	QString tooltest_logfile;
 	QProcess *ModprobeProcess;
 	QProcess *createProcess;
 	QProcess *NetworkDeviceTestProcess;
 	QProcess *toolsTestProcess;
 	QProcess *OpenvpnCiphersProcess;
+	QProcess *OpenvpnDigestProcess;
 	QProcess *NameAndPidOfProgramListenProcess;
+	QProcess *GetEmailAddressOfCertProcess;
+	QProcess *GetSmartcardSlotsProcess;
+	QProcess *GetSmartcardCertsFromSlotProcess;
+	QProcess *NeedsPassphraseProcess;
+	
 	QString resolvedIP;
 	bool resolveFinished;
 	QValueList<QHostAddress> list;
@@ -89,9 +110,18 @@ private:
 	KVpncConfig *config;
 	KTempFile *tmpfile;
 	bool retrieveValidNetworkdevice;
+	bool Pkcs11CertFound;
 	QStringList OpenvpnCiphers;
+	QStringList OpenvpnDigests;
+	QStringList KernelCrypto;
 	QString NameAndPidOfProgramListen;
+	QString EmailAddressOfCert;
+	QString OpenvpnDigestString;
+	QString IdType;
+	QStringList SmartcardSlots;
+	QStringList SmartcardCertsFromSlot;
 	int ListenPort;
+	int OpenvpnDigestCount;
 };
 
 #endif
