@@ -36,7 +36,9 @@ VpnAccountData::VpnAccountData( ConnectionType ConnType,const QString& Name)
 	ID="";
 	Username="";
 	UserPassword="";
-	IkeGroup="";
+	IkeGroup="modp2048";
+	EncryptionAlgorithm="3des";
+	AuthenticationAlgorithm="hmac_md5";
 	PerfectForwardSecrecy="";
 	NtDomainName="";
 	ApplicationVersion="";
@@ -60,6 +62,10 @@ VpnAccountData::VpnAccountData( ConnectionType ConnType,const QString& Name)
 	HttpProxyAuthType="";
 	HttpProxyUser="";
 	HttpProxyPass="";
+	RightNextHop="";
+	LeftNextHop="";
+	AuthMethod="chap";
+	TunnelDeviceType="tun";
 	saveUserPassword=false;
 	savePsk=false;
 	useAdvancedSettings=false;
@@ -163,7 +169,7 @@ VpnAccountData::VpnAccountData( ConnectionType ConnType,const QString& Name)
 	useTlsRemoteHost=false;
 	dontSaveUsername=false;
 	disableLzoCompression=false;
-	disableOpportunisticEncryption=false;
+	disableOpportunisticEncryption=true;
 	useMssfix=false;
 	useFragment=false;
 	disableCcp=false;
@@ -171,6 +177,13 @@ VpnAccountData::VpnAccountData( ConnectionType ConnType,const QString& Name)
 	savePrivateKeyPassword=false;
 	useHttpProxyAuth=false;
 	useOnlyCaCertAndUserAuth=false;
+	useMailAddressAsIdentifier=false;
+	useRightNextHop=false;
+	useLeftNextHop=false;
+	usePkcs11Providers=false;
+	allowEmptyPrivateKeyPassword=false;
+	useSmartcard=false;
+	allowIpAddressChangeOfPeer=false;
 
 	PeerTimeout=60;
 	ReconnectDelay=5;
@@ -185,6 +198,7 @@ VpnAccountData::VpnAccountData( ConnectionType ConnType,const QString& Name)
 	HttpProxyTimeout=100;
 	Mssfix=1300;
 	Fragment=1300;
+	CommandAfterConnectDelayTime=3;
 
 }
 
@@ -242,6 +256,16 @@ QString VpnAccountData::getIkeGroup() const
 	return IkeGroup;
 }
 
+QString VpnAccountData::getEncryptionAlgorithm() const
+{
+	return EncryptionAlgorithm;
+}
+
+QString VpnAccountData::getAuthenticationAlgorithm() const
+{
+	return AuthenticationAlgorithm;
+}
+
 QString VpnAccountData::getPerfectForwardSecrety() const
 {
 	return PerfectForwardSecrecy;
@@ -776,9 +800,9 @@ bool VpnAccountData::getDisableCcp() const
 	return disableCcp;
 }
 
-bool VpnAccountData::getUseSmardcard() const
+bool VpnAccountData::getUseSmartcard() const
 {
-	return useSmardcard;
+	return useSmartcard;
 }
 
 bool VpnAccountData::getUseXauthInteractive() const
@@ -796,19 +820,16 @@ bool VpnAccountData::getUseHttpProxyAuth() const
 	return useHttpProxyAuth;
 }
 
-
 QString VpnAccountData::getHttpProxyAuthType() const
 {
 	return HttpProxyAuthType;
 }
 
-
 QString VpnAccountData::getHttpProxyUser() const
 {
 	return HttpProxyUser;
 }
 
-
 QString VpnAccountData::getHttpProxyPass() const
 {
 	return HttpProxyPass;
@@ -819,6 +840,75 @@ bool VpnAccountData::getUseOnlyCaCertAndUserAuth() const
 	return useOnlyCaCertAndUserAuth;
 }
 
+bool VpnAccountData::getUseMailAddressAsIdentifier() const
+{
+	return useMailAddressAsIdentifier;
+}
+
+bool VpnAccountData::getUseRightNextHop() const
+{
+	return useRightNextHop;
+}
+
+bool VpnAccountData::getUseLeftNextHop() const
+{
+	return useLeftNextHop;
+}
+
+QString VpnAccountData::getPkcs11Providers() const
+{
+	return Pkcs11Providers;
+}
+
+QString VpnAccountData::getPkcs11SlotType() const
+{
+	return Pkcs11SlotType;
+}
+
+QString VpnAccountData::getPkcs11Slot() const
+{
+	return Pkcs11Slot;
+}
+
+QString VpnAccountData::getPkcs11IdType() const
+{
+	return Pkcs11IdType;
+}
+
+QString VpnAccountData::getPkcs11Id() const
+{
+	return Pkcs11Id;
+}
+
+QString VpnAccountData::getPkcs11SignMode() const
+{
+	return Pkcs11SignMode;
+}
+
+bool VpnAccountData::getUsePkcs11Providers() const
+{
+	return usePkcs11Providers;
+}
+
+bool VpnAccountData::getAllowEmptyPrivateKeyPassword() const
+{
+	return allowEmptyPrivateKeyPassword;
+}
+
+bool VpnAccountData::getAllowIpAddressChangeOfPeer() const
+{
+	return allowIpAddressChangeOfPeer;
+}
+
+QString VpnAccountData::getAuthMethod() const
+{
+	return AuthMethod;
+}
+
+int VpnAccountData::getCommandAfterConnectDelayTime() const
+{
+	return CommandAfterConnectDelayTime;
+}
 
 /* set methods */
 void VpnAccountData::setConnectionType( ConnectionType type )
@@ -866,6 +956,16 @@ void VpnAccountData::setIkeGroup( const QString &IkeGroup )
 	this->IkeGroup = IkeGroup;
 }
 
+void VpnAccountData::setEncryptionAlgorithm( const QString &EncryptionAlgorithm )
+{
+	this->EncryptionAlgorithm = EncryptionAlgorithm;
+}
+
+void VpnAccountData::setAuthenticationAlgorithm( const QString &AuthenticationAlgorithm)
+{
+	this->AuthenticationAlgorithm= AuthenticationAlgorithm;
+}
+
 void VpnAccountData::setPerfectForwardSecrety( const QString &PerfectForwardSecrecy )
 {
 	this->PerfectForwardSecrecy = PerfectForwardSecrecy;
@@ -1402,9 +1502,9 @@ void VpnAccountData::setDisableCcp( bool disableCcp )
 	this->disableCcp = disableCcp;
 }
 
-void VpnAccountData::setUseSmardcard( bool useSmardcard)
+void VpnAccountData::setUseSmartcard( bool useSmartcard)
 {
-	this->useSmardcard = useSmardcard;
+	this->useSmartcard = useSmartcard;
 }
 
 void VpnAccountData::setUseXauthInteractive( bool useXauthInteractive)
@@ -1441,3 +1541,74 @@ void VpnAccountData::setUseOnlyCaCertAndUserAuth(bool useOnlyCaCertAndUserAuth)
 {
 	this->useOnlyCaCertAndUserAuth = useOnlyCaCertAndUserAuth;
 }
+
+void VpnAccountData::setUseMailAddressAsIdentifier(bool useMailAddressAsIdentifier )
+{
+	this->useMailAddressAsIdentifier = useMailAddressAsIdentifier;
+}
+
+void VpnAccountData::setUseRightNextHop( bool useRightNextHop )
+{
+	this->useRightNextHop = useRightNextHop;
+}
+
+void VpnAccountData::setUseLeftNextHop( bool useLeftNextHop )
+{
+	this->useLeftNextHop = useLeftNextHop;
+}
+
+void VpnAccountData::setPkcs11Providers( QString Pkcs11Providers)
+{
+	this->Pkcs11Providers = Pkcs11Providers;
+}
+
+void VpnAccountData::setPkcs11SlotType(QString Pkcs11SlotType)
+{
+	this->Pkcs11SlotType = Pkcs11SlotType;
+}
+
+void VpnAccountData::setPkcs11Slot(QString Pkcs11Slot)
+{
+	this->Pkcs11Slot = Pkcs11Slot;
+}
+
+void VpnAccountData::setPkcs11IdType(QString Pkcs11IdType)
+{
+	this->Pkcs11IdType = Pkcs11IdType;
+}
+
+void VpnAccountData::setPkcs11Id(QString Pkcs11Id)
+{
+	this->Pkcs11Id = Pkcs11Id;
+}
+
+void VpnAccountData::setPkcs11SignMode(QString Pkcs11SignMode)
+{
+	this->Pkcs11SignMode = Pkcs11SignMode;
+}
+
+void VpnAccountData::setUsePkcs11Providers(bool usePkcs11Providers)
+{
+	this->usePkcs11Providers = usePkcs11Providers;
+}
+
+void VpnAccountData::setAllowEmptyPrivateKeyPassword( bool allowEmptyPrivateKeyPassword)
+{
+	this->allowEmptyPrivateKeyPassword = allowEmptyPrivateKeyPassword;
+}
+
+void VpnAccountData::setAllowIpAddressChangeOfPeer( bool allowIpAddressChangeOfPeer )
+{
+	this->allowIpAddressChangeOfPeer = allowIpAddressChangeOfPeer;
+}
+
+void VpnAccountData::setAuthMethod( QString AuthMethod )
+{
+	this->AuthMethod = AuthMethod;
+}
+
+void VpnAccountData::setCommandAfterConnectDelayTime( int CommandAfterConnectDelayTime)
+{
+	this->CommandAfterConnectDelayTime = CommandAfterConnectDelayTime;
+}
+
