@@ -24,10 +24,6 @@
 #include <qobject.h>
 #include <qstringlist.h>
 
-//enum connectiontypes{vpnc,racoon,other};
-
-
-
 /**
 * holds the data for a account
 * @author Christoph Thielecke
@@ -35,8 +31,9 @@
 class VpnAccountData : public QObject
 {
 public:
-// 	enum ConnectionType {cisco=0, racoon=1, freeswan=2, pptp=3,openvpn=4,l2tpd_racoon=5,l2tpd_freeswan=6,other=7};
-	enum ConnectionType {cisco=0, racoon=1, freeswan=2, pptp=3,openvpn=4,other=5};
+	enum ConnectionType {cisco=0, racoon=1, freeswan=2, pptp=3,openvpn=4,l2tpd_racoon=5,l2tpd_freeswan=6,other=7};
+// 	enum ConnectionType {cisco=0, racoon=1, freeswan=2, pptp=3,openvpn=4,l2tpd_racoon=5,other=6};
+// 	enum ConnectionType {cisco=0, racoon=1, freeswan=2, pptp=3,openvpn=4,other=5};
 	enum AuthenticationType {cert=0, psk=1};
 
 	/**
@@ -60,6 +57,8 @@ public:
 	QString getUserName() const; //< returns UserName
 	QString getUserPassword() const; //< returns UserPassword
 	QString getIkeGroup() const; //< returns IkeGroup
+	QString getEncryptionAlgorithm() const; //< returns EncryptionAlgorithm
+	QString getAuthenticationAlgorithm() const; //< returns AuthenticationAlgorithm
 	QString getPerfectForwardSecrety() const; //< returns PerfectForwardSecrecy
 	QString getNtDomainName() const; //< returns NtDomainName
 	QString getApplicationVersion() const; //< returns ApplicationVersion
@@ -98,6 +97,13 @@ public:
 	QString getHttpProxyAuthType() const;
 	QString getHttpProxyUser() const;
 	QString getHttpProxyPass() const;
+	QString getPkcs11Providers() const;
+	QString getPkcs11SlotType() const;
+	QString getPkcs11Slot() const;
+	QString getPkcs11IdType() const;
+	QString getPkcs11Id() const;
+	QString getPkcs11SignMode() const;
+	QString getAuthMethod() const;
 
 	int getLocalPort() const; //< returns LocalPort
 	int getRemotePort() const; //< returns RemotePort
@@ -113,6 +119,7 @@ public:
 	int getMssfix() const;
 	int getFragment() const;
 	int getPeerTimeout() const;
+	int getCommandAfterConnectDelayTime() const;
 	AuthenticationType getAuthType() const; //< returns AuthType
 	QStringList getAdditionalNetworkRoutes() const;
 	bool getSaveUserPassword() const; //< returns saveUserPassword
@@ -173,11 +180,19 @@ public:
 	bool getUseMssfix() const;
 	bool getUseFragment() const;
 	bool getDisableCcp() const;
-	bool getUseSmardcard() const;
+	bool getUseSmartcard() const;
 	bool getUseXauthInteractive() const;
 	bool getSavePrivateKeyPassword() const;
 	bool getUseHttpProxyAuth () const;
 	bool getUseOnlyCaCertAndUserAuth() const;
+	bool getUseMailAddressAsIdentifier() const;
+	bool getUseRightNextHop() const;
+	bool getUseLeftNextHop() const;
+	bool getUsePkcs11Providers() const;
+	bool getAllowEmptyPrivateKeyPassword() const;
+	bool getAllowIpAddressChangeOfPeer() const;
+
+
 
 	/**
 	 *   Sets the connection type
@@ -238,6 +253,8 @@ public:
 	 * @param IkeGroup The group
 	 */
 	void setIkeGroup( const QString &IkeGroup );
+	void setEncryptionAlgorithm( const QString &EncryptionAlgorithm );
+	void setAuthenticationAlgorithm( const QString &AuthenticationAlgorithm );
 
 	void setPerfectForwardSecrety( const QString &PerfectForwardSecrecy );
 	void setNtDomainName( const QString &Name );
@@ -337,7 +354,7 @@ public:
 	void setReconnectDelay( int ReconnectDelay );
 	void setUseTlsRemoteHost( bool useTlsRemoteHost );
 	void setTlsRemoteHost( QString TlsRemoteHost );
-	void setDisableOpportunisticEncryption( bool disableOpportunisticEncryption );
+	void setDisableOpportunisticEncryption( bool );
 	void setRightNextHop( QString RightNextHop );
 	void setLeftNextHop( QString LeftNextHop );
 	void setUseMssfix( bool useMssfix );
@@ -346,7 +363,7 @@ public:
 	void setFragment( int Fragment );
 	void setPeerTimeout( int PeerTimeout );
 	void setDisableCcp( bool disableCcp );
-	void setUseSmardcard( bool useSmardcard );
+	void setUseSmartcard( bool useSmartcard );
 	void setUseXauthInteractive( bool useXauthInteractive );
 	void setSavePrivateKeyPassword ( bool savePrivateKeyPassword );
 	void setUseHttpProxyAuth (bool useHttpProxyAuth );
@@ -354,7 +371,20 @@ public:
 	void setHttpProxyUser(QString HttpProxyUser );
 	void setHttpProxyPass(QString HttpProxyPass );
 	void setUseOnlyCaCertAndUserAuth(bool useOnlyCaCertAndUserAuth );
-
+	void setUseMailAddressAsIdentifier(bool useMailAddressAsIdentifier );
+	void setUseRightNextHop(bool useRightNextHop);
+	void setUseLeftNextHop (bool useLeftNextHop);
+	void setPkcs11Providers(QString Pkcs11Providers);
+	void setPkcs11SlotType(QString Pkcs11SlotType);
+	void setPkcs11Slot(QString Pkcs11Slot);
+	void setPkcs11IdType(QString Pkcs11IdType);
+	void setPkcs11Id(QString Pkcs11Id);
+	void setPkcs11SignMode(QString Pkcs11SignMode);
+	void setUsePkcs11Providers(bool usePkcs11Providers);
+	void setAllowEmptyPrivateKeyPassword (bool allowEmptyPrivateKeyPassword );
+	void setAllowIpAddressChangeOfPeer( bool allowIpAddressChangeOfPeer );
+	void setAuthMethod( QString AuthMethod );
+	void setCommandAfterConnectDelayTime( int CommandAfterConnectDelayTime );
 
 protected:
 	QString Name; //< Name of the profile
@@ -362,8 +392,10 @@ protected:
 	QString ID; //< ID of remote gateway
 	QString Username; //< Username (cisco)
 	QString UserPassword; //< User password (cisco)
-	QString IkeGroup; //< DH group for PFS (cisco)
-	QString PerfectForwardSecrecy; //< Type of PFS (cisco)
+	QString IkeGroup; //< DH group for PFS (cisco,racoon)
+	QString PerfectForwardSecrecy; //< Type of PFS (cisco, racoon)
+	QString EncryptionAlgorithm; // phase 2 (ipsec)
+	QString AuthenticationAlgorithm; // phase 2 (ipsec)
 	QString NtDomainName; //< NT domain name for authentication (cisco/pptp)
 	QString ApplicationVersion; //< userdefined application version (cisco)
 	QString RemoteNetAddr; //< Remote network addr of VPN
@@ -403,6 +435,13 @@ protected:
 	QString HttpProxyAuthType;
 	QString HttpProxyUser;
 	QString HttpProxyPass;
+	QString Pkcs11Providers;
+	QString Pkcs11SlotType;
+	QString Pkcs11Slot;
+	QString Pkcs11IdType;
+	QString Pkcs11Id;
+	QString Pkcs11SignMode;
+	QString AuthMethod; //< authorization method for pptp (chap/pap)
 
 	QStringList AdditionalNetworkRoutes;
 
@@ -464,11 +503,17 @@ protected:
 	bool useMssfix;
 	bool useFragment;
 	bool disableCcp;
-	bool useSmardcard;
+	bool useSmartcard;
 	bool useXauthInteractive;
 	bool savePrivateKeyPassword;
 	bool useHttpProxyAuth;
 	bool useOnlyCaCertAndUserAuth; //< use only a CA certificate and authentication with username and password (openvpn)
+	bool useMailAddressAsIdentifier; //< x509 cert
+	bool useRightNextHop;
+	bool useLeftNextHop;
+	bool usePkcs11Providers;
+	bool allowEmptyPrivateKeyPassword;
+	bool allowIpAddressChangeOfPeer;
 	int Mtu;
 	int Mru;
 	int UdpPort; //< Port number of UDP port
@@ -483,6 +528,7 @@ protected:
 	int Mssfix;
 	int Fragment;
 	int PeerTimeout; // timeout given from peer (cisco for example gives that)
+	int CommandAfterConnectDelayTime; //< time which is wait after connect before the cmd after connect should be executed
 
 public:
 	ConnectionType connectiontype; //< Type of the connection (cisco, freeswan, racoon, etc.)
